import transaction
import sqlalchemy as sqla
from ptah import config, form
from pyramid.httpexceptions import HTTPForbidden

from base import Base


class TestTypeInfo(Base):

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        super(TestTypeInfo, self).tearDown()

    def _setup_ptah(self):
        pass

    def test_tinfo(self):
        import ptah.cms

        global MyContent
        class MyContent(ptah.cms.Content):

            __type__ = ptah.cms.Type('mycontent', 'MyContent')

        self._init_ptah()

        all_types = ptah.cms.get_types()

        self.assertTrue('cms-type:mycontent' in all_types)

        tinfo = ptah.cms.get_type('cms-type:mycontent')

        self.assertEqual(tinfo.__uri__, 'cms-type:mycontent')
        self.assertEqual(tinfo.name, 'mycontent')
        self.assertEqual(tinfo.title, 'MyContent')
        self.assertEqual(tinfo.cls, MyContent)

    def test_tinfo_title(self):
        import ptah.cms

        class MyContent(ptah.cms.Content):
            __type__ = ptah.cms.Type('mycontent')

        self.assertEqual(MyContent.__type__.title, 'Mycontent')

        class MyContent(ptah.cms.Content):
            __type__ = ptah.cms.Type('mycontent', 'MyContent')

        self.assertEqual(MyContent.__type__.title, 'MyContent')

    def test_tinfo_checks(self):
        import ptah.cms

        global MyContent, MyContainer
        class MyContent(ptah.cms.Content):
            __type__ = ptah.cms.Type('mycontent', 'Content', permission=None)
        class MyContainer(ptah.cms.Container):
            __type__ = ptah.cms.Type('mycontainer', 'Container')
        self._init_ptah()

        content = MyContent()
        container = MyContainer()

        # always fail
        self.assertFalse(MyContent.__type__.is_allowed(content))
        self.assertRaises(
            HTTPForbidden, MyContent.__type__.check_context, content)

        #
        self.assertTrue(MyContent.__type__.is_allowed(container))
        self.assertEqual(MyContent.__type__.check_context(container), None)

        # permission
        MyContent.__type__.permission = 'Protected'
        self.assertFalse(MyContent.__type__.is_allowed(container))
        self.assertRaises(
            HTTPForbidden, MyContent.__type__.check_context, container)

    def test_tinfo_list(self):
        import ptah.cms

        global MyContent, MyContainer
        class MyContent(ptah.cms.Content):
            __type__ = ptah.cms.Type('mycontent', 'Content', permission=None)
        class MyContainer(ptah.cms.Container):
            __type__ = ptah.cms.Type('mycontainer', 'Container')
        self._init_ptah()

        content = MyContent()
        container = MyContainer()

        self.assertEqual(MyContent.__type__.list_types(content), ())
        self.assertEqual(MyContent.__type__.list_types(container), ())

        self.assertEqual(MyContainer.__type__.list_types(container),
                         [MyContent.__type__])

        MyContent.__type__.global_allow = False
        self.assertEqual(MyContainer.__type__.list_types(container), [])

        MyContent.__type__.global_allow = True
        MyContent.__type__.permission = 'Protected'
        self.assertEqual(MyContainer.__type__.list_types(container), [])

    def test_tinfo_global_allow_Node(self):
        import ptah.cms

        global MyContent
        class MyContent(ptah.cms.Node):
            __type__ = ptah.cms.Type('mycontent', 'Content', permission=None)
        class MyContainer(ptah.cms.Node):
            __type__ = ptah.cms.Type('mycontainer', 'Container',
                                     global_allow = True)
        self._init_ptah()

        self.assertFalse(MyContent.__type__.global_allow)
        self.assertTrue(MyContainer.__type__.global_allow)

    def test_tinfo_list_filtered(self):
        import ptah.cms

        global MyContent, MyContainer
        class MyContent(ptah.cms.Content):
            __type__ = ptah.cms.Type('mycontent', 'Content', permission=None)
        class MyContainer(ptah.cms.Container):
            __type__ = ptah.cms.Type(
                'mycontainer', 'Container', filter_content_types=True)

        self._init_ptah()

        container = MyContainer()
        self.assertEqual(MyContainer.__type__.list_types(container), [])

        MyContainer.__type__.allowed_content_types = ('mycontent',)
        self.assertEqual(MyContainer.__type__.list_types(container),
                         [MyContent.__type__])

    def test_tinfo_conflicts(self):
        import ptah.cms

        global MyContent
        class MyContent(ptah.cms.Content):
            __type__ = ptah.cms.Type('mycontent2', 'MyContent')
        class MyContent2(ptah.cms.Content):
            __type__ = ptah.cms.Type('mycontent2', 'MyContent')

        self.assertRaises(
            config.ConflictError, self._init_ptah)

    def test_tinfo_create(self):
        import ptah.cms

        global MyContent
        class MyContent(ptah.cms.Content):
            __type__ = ptah.cms.Type('mycontent', 'MyContent')

        self._init_ptah()

        all_types = ptah.cms.get_types()

        content = all_types['cms-type:mycontent'].create(title='Test content')

        self.assertTrue(isinstance(content, MyContent))
        self.assertEqual(content.title, 'Test content')
        self.assertTrue(content not in ptah.cms.Session)

    def test_tinfo_alchemy(self):
        import ptah.cms

        global MyContent
        class MyContent(ptah.cms.Content):
            __tablename__ = "test_mycontents"
            __type__ = ptah.cms.Type('mycontent', 'MyContent')

        self._init_ptah()

        self.assertEqual(
            MyContent.__mapper_args__['polymorphic_identity'],
            'cms-type:mycontent')

        self.assertTrue(
            MyContent.__uri_factory__().startswith('cms-mycontent:'))

    def test_tinfo_resolver(self):
        import ptah, ptah.cms

        global MyContent
        class MyContent(ptah.cms.Content):
            __type__ = ptah.cms.Type('mycontent2', 'MyContent')

        self._init_ptah()

        content = MyContent.__type__.create(title='Test content')
        c_uri = content.__uri__
        ptah.cms.Session.add(content)
        transaction.commit()

        c = ptah.resolve(c_uri)
        self.assertTrue(isinstance(c, MyContent))

    def test_tinfo_fieldset(self):
        import ptah, ptah.cms

        MySchema = ptah.cms.ContentSchema + \
            form.Fieldset(form.TextField('test'))

        class MyContent(ptah.cms.Content):
            __type__ = ptah.cms.Type('mycontent2', 'MyContent',
                                     fieldset=MySchema)

        tinfo = MyContent.__type__
        self.assertIs(tinfo.fieldset, MySchema)

    def test_tinfo_fieldset_gen(self):
        import ptah, ptah.cms

        global MyContent
        class MyContent(ptah.cms.Content):
            __tablename__ = 'test_content'
            __type__ = ptah.cms.Type('mycontent2', 'MyContent')

            test = sqla.Column(sqla.Unicode())

        self._init_ptah()

        tinfo = MyContent.__type__
        self.assertIn('test', tinfo.fieldset)
        self.assertIn('title', tinfo.fieldset)
        self.assertIn('description', tinfo.fieldset)
        self.assertNotIn('__owner__', tinfo.fieldset)
        self.assertEqual(len(tinfo.fieldset), 3)

    def test_tinfo_type_resolver(self):
        import ptah, ptah.cms

        global MyContent
        class MyContent(ptah.cms.Content):
            __type__ = ptah.cms.Type('mycontent2', 'MyContent')

        self._init_ptah()

        tinfo_uri = MyContent.__type__.__uri__

        self.assertEqual(tinfo_uri, 'cms-type:mycontent2')
        self.assertIs(ptah.resolve(tinfo_uri), MyContent.__type__)

    def test_names_filter(self):
        from ptah.cms.tinfo import names_filter

        self.assertFalse(names_filter('_test'))
        self.assertFalse(names_filter('__test__'))
        self.assertTrue(names_filter('__test__', ('__test__',)))

        excludeNames = ('expires', 'contributors', 'creators',
                        'view', 'subjects',
                        'publisher', 'effective', 'created', 'modified')
        for name in excludeNames:
            self.assertFalse(names_filter(name))
            self.assertTrue(names_filter(name, (name,)))



import ptah

View = ptah.Permission('ptah:View', 'View')
AddContent = ptah.Permission('ptah-cms:AddContent', 'Add content')
ModifyContent = ptah.Permission('ptah-cms:EditContent', 'Modify content')
DeleteContent = ptah.Permission('ptah-cms:DeleteContent', 'Delete content')
ShareContent = ptah.Permission('ptah-cms:ShareContent', 'Share content')



import transaction
import sqlahelper
import sqlalchemy as sqla
from ptah import form

from base import Base

SqlaBase = sqlahelper.get_base()
Session = sqlahelper.get_session()


class TestSqlSchema(Base):

    def test_sqlschema_fields(self):
        import ptah

        class Test(SqlaBase):
            __tablename__ = 'test'

            id = sqla.Column('id', sqla.Integer, primary_key=True)
            name = sqla.Column(sqla.Unicode())
            count = sqla.Column(sqla.Integer())
            score = sqla.Column(sqla.Float())
            date = sqla.Column(sqla.Date())
            datetime = sqla.Column(sqla.DateTime())
            boolean = sqla.Column(sqla.Boolean())

        fieldset = ptah.generate_fieldset(Test)

        # no primary keya
        self.assertNotIn('id', fieldset)

        self.assertEqual(fieldset['name'].__field__, 'text')
        self.assertEqual(fieldset['count'].__field__, 'int')
        self.assertEqual(fieldset['score'].__field__, 'float')
        self.assertEqual(fieldset['date'].__field__, 'date')
        self.assertEqual(fieldset['datetime'].__field__, 'datetime')
        self.assertEqual(fieldset['boolean'].__field__, 'bool')
        self.assertEqual(fieldset['name'].title, 'Name')

        fieldset = ptah.generate_fieldset(Test, fieldNames=('name', 'count'))
        self.assertEqual(len(fieldset), 2)
        self.assertIn('name', fieldset)
        self.assertIn('count', fieldset)

        fieldset = ptah.generate_fieldset(
            Test, fieldNames=('id', 'name'), skipPrimaryKey=False)
        self.assertEqual(len(fieldset), 2)
        self.assertIn('name', fieldset)
        self.assertIn('id', fieldset)
        self.assertTrue(fieldset['id'].readonly)

        # no table
        class TestNoTable(Test):
            pass

        fieldset = ptah.generate_fieldset(TestNoTable)
        self.assertEqual(len(fieldset), 6)

    def test_sqlschema_extra_fields(self):
        import ptah

        class Test2(SqlaBase):
            __tablename__ = 'test2'

            id = sqla.Column('id', sqla.Integer, primary_key=True)
            name = sqla.Column(
                sqla.Unicode(),
                info={'title': 'Test title',
                      'missing': 'missing value',
                      'description': 'Description',
                      'field_type': 'textarea',
                      'vocabulary': ['1','2']})

        fieldset = ptah.generate_fieldset(Test2)

        field = fieldset['name']

        self.assertEqual(field.title, 'Test title')
        self.assertEqual(field.description, 'Description')
        self.assertEqual(field.missing, 'missing value')
        self.assertEqual(field.__field__, 'textarea')
        self.assertEqual(field.vocabulary, ['1', '2'])

    def test_sqlschema_custom(self):
        import ptah

        field = form.TextField('name', title = 'Custom')

        class Test3(SqlaBase):
            __tablename__ = 'test3'
            id = sqla.Column('id', sqla.Integer, primary_key=True)
            name = sqla.Column(sqla.Unicode(), info={'field': field})

        fieldset = ptah.generate_fieldset(Test3)

        m_field = fieldset['name']

        self.assertEqual(m_field.name, 'name')
        self.assertEqual(m_field.title, 'Custom')
        self.assertIs(m_field, field)

    def test_sqlschema_unknown(self):
        import ptah

        class Test2(SqlaBase):
            __tablename__ = 'test5'

            id = sqla.Column('id', sqla.Integer, primary_key=True)
            name = sqla.Column(sqla.Unicode())
            json = sqla.Column(ptah.JsonListType())

        fieldset = ptah.generate_fieldset(Test2)

        self.assertNotIn('json', fieldset)


class TestQueryFreezer(Base):

    def test_freezer_one(self):
        import ptah

        class Test(SqlaBase):
            __tablename__ = 'test10'

            id = sqla.Column('id', sqla.Integer, primary_key=True)
            name = sqla.Column(sqla.Unicode())

        SqlaBase.metadata.create_all()
        transaction.commit()

        sql_get = ptah.QueryFreezer(
            lambda: Session.query(Test)
            .filter(Test.name == sqla.sql.bindparam('name')))

        self.assertRaises(
            sqla.orm.exc.NoResultFound, sql_get.one, name='test')

        rec = Test()
        rec.name = 'test'
        Session.add(rec)
        Session.flush()

        rec = sql_get.one(name='test')
        self.assertEqual(rec.name, 'test')


        rec = Test()
        rec.name = 'test'
        Session.add(rec)
        Session.flush()

        self.assertRaises(
            sqla.orm.exc.MultipleResultsFound, sql_get.one, name='test')

    def test_freezer_first(self):
        import ptah

        class Test(SqlaBase):
            __tablename__ = 'test12'

            id = sqla.Column('id', sqla.Integer, primary_key=True)
            name = sqla.Column(sqla.Unicode())

        SqlaBase.metadata.create_all()
        transaction.commit()

        sql_get = ptah.QueryFreezer(
            lambda: Session.query(Test)
            .filter(Test.name == sqla.sql.bindparam('name')))

        self.assertIsNone(sql_get.first(name='test'))

        rec = Test()
        rec.name = 'test'
        Session.add(rec)
        Session.flush()

        rec = sql_get.one(name='test')
        self.assertEqual(rec.name, 'test')

        sql_get.reset()
        rec = sql_get.one(name='test')
        self.assertEqual(rec.name, 'test')


class TestJsonDict(Base):

    def test_jsondict(self):
        import ptah

        class Test(SqlaBase):
            __tablename__ = 'test14'

            id = sqla.Column('id', sqla.Integer, primary_key=True)
            data = sqla.Column(ptah.JsonDictType())

        SqlaBase.metadata.create_all()
        transaction.commit()

        rec = Test()
        rec.data = {'test': 'val'}
        Session.add(rec)
        Session.flush()
        id = rec.id
        transaction.commit()

        rec = Session.query(Test).filter_by(id = id).one()
        self.assertEqual(rec.data, {'test': 'val'})

        rec.data['test2'] = 'val2'
        transaction.commit()

        rec = Session.query(Test).filter_by(id = id).one()
        self.assertEqual(rec.data,
                         {'test': 'val', 'test2': 'val2'})

        del rec.data['test']
        transaction.commit()

        rec = Session.query(Test).filter_by(id = id).one()
        self.assertEqual(rec.data, {'test2': 'val2'})


class TestJsonList(Base):

    def test_jsonlist(self):
        import ptah

        class Test(SqlaBase):
            __tablename__ = 'test15'

            id = sqla.Column('id', sqla.Integer, primary_key=True)
            data = sqla.Column(ptah.JsonListType())

        SqlaBase.metadata.create_all()
        transaction.commit()

        rec = Test()
        rec.data = ['test']
        Session.add(rec)
        Session.flush()
        id = rec.id
        transaction.commit()

        rec = Session.query(Test).filter_by(id = id).one()
        self.assertEqual(rec.data, ['test'])

        rec.data[0] = 'test2'
        transaction.commit()

        rec = Session.query(Test).filter_by(id = id).one()
        self.assertEqual(rec.data, ['test2'])

        rec.data.append('test')
        transaction.commit()

        rec = Session.query(Test).filter_by(id = id).one()
        self.assertEqual(rec.data, ['test2', 'test'])

        del rec.data[rec.data.index('test2')]
        transaction.commit()

        rec = Session.query(Test).filter_by(id = id).one()
        self.assertEqual(rec.data, ['test'])



import colander
import os
import shutil
import tempfile
import unittest

from pyramid.testing import setUp, tearDown
from zope.interface.registry import Components
from zope.interface.interface import InterfaceClass
from zope.interface.interfaces import IObjectEvent

import ptah
from ptah.config.api import objectEventNotify
from ptah.config.settings import get_settings


class BaseTesting(unittest.TestCase):

    def _init_ptah(self, settings={}, *args, **kw):
        ptah.config.initialize(
            self.config, ('ptah.config', self.__class__.__module__),
            initsettings=False)

    def setUp(self):
        self.config = setUp()
        self.registry = self.config.registry

    def tearDown(self):
        ptah.config.cleanup_system(self.__class__.__module__)
        tearDown()


class TestSettings(BaseTesting):

    def test_settings_group_uninitialized(self):
        node = ptah.config.SchemaNode(
                colander.Str(),
                name = 'node',
                default = 'test')

        group = ptah.config.register_settings(
            'group1', node,
            title = 'Section title',
            description = 'Section description',
            )

        self.assertEqual(group.get('node'), 'test')

    def test_settings_register_errs(self):
        self.assertRaises(
            RuntimeError,
            ptah.config.register_settings,
            'section',
            colander.SchemaNode(
                colander.Str(),
                name = 'node')
            )

    def test_settings_register_simple(self):
        node = ptah.config.SchemaNode(
                colander.Str(),
                name = 'node',
                default = 'test')

        group = ptah.config.register_settings(
            'group1', node,
            title = 'Section title',
            description = 'Section description',
            )

        self.assertEqual(group.name, 'group1')
        self.assertEqual(group.title, 'Section title')
        self.assertEqual(group.description, 'Section description')
        self.assertEqual(len(group.schema.children), 1)
        self.assertTrue(isinstance(group.category, InterfaceClass))

        self._init_ptah()

        self.assertEqual(len(group.schema.children), 1)
        self.assertTrue(group.schema.children[0] is node)
        self.assertEqual(group.node, 'test')
        self.assertEqual(group['node'], 'test')
        self.assertRaises(
            AttributeError,
            group.__getattr__, 'unknown')
        self.assertRaises(
            KeyError,
            group.__getitem__, 'unknown')

        group.node = 'test2'
        self.assertFalse(group.node == group['node'])

    def test_settings_group_validation(self):
        def validator(node, appstruct):
            raise colander.Invalid(node['node'], 'Error')

        node = ptah.config.SchemaNode(
                colander.Str(),
                name = 'node',
                default = 'test')

        group = ptah.config.register_settings(
            'group2', node, validator=validator)

        self._init_ptah()

        try:
            group.schema.deserialize({'group2.node': 'value'})
        except colander.Invalid, err:
            pass

        self.assertEqual(err.asdict(), {'group2.node': 'Error'})

    def test_settings_group_multiple_validation(self):
        def validator1(node, appstruct):
            raise colander.Invalid(node['node1'], 'Error1')

        def validator2(node, appstruct):
            raise colander.Invalid(node, 'Error2')

        node1 = ptah.config.SchemaNode(
                colander.Str(),
                name = 'node1',
                default = 'test')

        node2 = ptah.config.SchemaNode(
                colander.Str(),
                name = 'node2',
                default = 'test')

        group = ptah.config.register_settings(
            'group3', node1, node2, validator=(validator1, validator2))

        self._init_ptah()

        try:
            group.schema.deserialize({
                    'group3.node1': 'value',
                    'group3.node2': 'value'})
        except colander.Invalid, err:
            pass

        self.assertEqual(err.asdict(),
                         {'group3.group3': 'Error2', 'group3.node1': 'Error1'})

    def test_settings_export(self):
        node1 = ptah.config.SchemaNode(
                colander.Str(),
                name = 'node1',
                default = 'test')

        node2 = ptah.config.SchemaNode(
                colander.Str(),
                name = 'node2',
                default = 'test')

        group = ptah.config.register_settings('group4', node1, node2)

        self._init_ptah()

        settings = get_settings()

        # changed settings
        self.assertEqual(settings.export(), {})

        # default settings
        self.assertEqual(settings.export(default=True),
                         {'group4.node1': 'test', 'group4.node2': 'test'})

        # changed settings
        group['node2'] = 'changed'
        data = dict(settings.export())
        self.assertEqual(data, {'group4.node2': 'changed'})

    def _create_default_group(self):
        node1 = ptah.config.SchemaNode(
                colander.Str(),
                name = 'node1',
                default = 'default1')

        node2 = ptah.config.SchemaNode(
                colander.Int(),
                name = 'node2',
                default = 10)

        group = ptah.config.register_settings('group', node1, node2)
        self._init_ptah()

        return group

    def test_settings_load_rawdata(self):
        group = self._create_default_group()

        get_settings()._load({'group.node1': 'val1'})

        # new value
        self.assertEqual(group['node1'], 'val1')

        # default value
        self.assertEqual(group['node2'], 10)

    def test_settings_load_rawdata_and_change_defaults(self):
        group = self._create_default_group()

        # change defaults
        get_settings()._load({'group.node2': 30}, setdefaults=True)

        # new values
        self.assertEqual(group['node1'], 'default1')
        self.assertEqual(group['node2'], 30)

        self.assertEqual(group.schema['node1'].default, 'default1')
        self.assertEqual(group.schema['node2'].default, 30)

    def test_settings_load_rawdata_with_errors_in_rawdata(self):
        group = self._create_default_group()

        get_settings()._load({10: 'value'})
        self.assertEqual(dict(group), {'node1': 'default1', 'node2': 10})

    def test_settings_load_rawdata_with_errors_in_values(self):
        group = self._create_default_group()

        get_settings()._load({'group.node2': 'vadfw234lue'})
        self.assertEqual(dict(group), {'node1': 'default1', 'node2': 10})

    def test_settings_load_defaults_rawdata_with_errors_in_values(self):
        group = self._create_default_group()

        get_settings()._load({'group.node2': 'vadfw234lue'}, setdefaults=True)
        self.assertEqual(dict(group),
                         {'node1': 'default1', 'node2': 10})

    def test_settings_load_defaults_rawdata_with_errors_in_colander(self):
        node = ptah.config.SchemaNode(
            ptah.config.Sequence(), colander.SchemaNode(colander.Str()),
            name = 'node1',
            default = ())

        group = ptah.config.register_settings('group', node)
        self._init_ptah()

        self.assertRaises(
            KeyError,
            get_settings()._load,
            {'group.node1.0': '1',
             'group.node1.3': '1'}, setdefaults=True)

    def test_settings_init_with_no_loader_with_defaults(self):
        group = self._create_default_group()

        get_settings().init({'group.node1': 'new-default',
                             'group.node2': 50})

        self.assertEqual(group['node1'], 'new-default')
        self.assertEqual(group['node2'], 50)

        self.assertEqual(group.schema['node1'].default, 'new-default')
        self.assertEqual(group.schema['node2'].default, 50)


class TestSettingsInitialization(BaseTesting):

    def setUp(self):
        ptah.config.cleanup_system()
        BaseTesting.setUp(self)
        self.dir = tempfile.mkdtemp()

    def tearDown(self):
        BaseTesting.tearDown(self)
        shutil.rmtree(self.dir)

    def test_settings_initialize_events(self):
        self._init_ptah()

        sm = self.config.registry

        events = []

        def h1(ev):
            events.append(ev)

        def h2(ev):
            events.append(ev)

        sm.registerHandler(h1, (ptah.config.SettingsInitializing,))
        sm.registerHandler(h2, (ptah.config.SettingsInitialized,))

        settings = get_settings()

        ptah.config.initialize_settings({}, self.config)

        self.assertTrue(isinstance(events[0], ptah.config.SettingsInitializing))
        self.assertTrue(isinstance(events[1], ptah.config.SettingsInitialized))

        self.assertTrue(events[0].config is self.config)
        self.assertTrue(events[1].config is self.config)

    def test_settings_initialize_events_exceptions(self):
        self._init_ptah()

        sm = self.config.registry

        events = []
        err = TypeError()

        def h1(ev):
            raise err

        sm.registerHandler(h1, (ptah.config.SettingsInitializing,))
        try:
            ptah.config.initialize_settings({}, self.config)
        except Exception, exc:
            pass

        self.assertIsInstance(exc, ptah.config.StopException)
        self.assertIs(exc.exc, err)

    def test_settings_initialize_only_once(self):
        self._init_ptah()
        ptah.config.initialize_settings({}, self.config)

        self.assertRaises(
            RuntimeError,
            ptah.config.initialize_settings, {}, self.config)

    def test_settings_initialize_load_default(self):
        node1 = ptah.config.SchemaNode(
                colander.Str(),
                name = 'node1',
                default = 'default1')

        node2 = ptah.config.SchemaNode(
                colander.Int(),
                name = 'node2',
                default = 10)

        group = ptah.config.register_settings('group', node1, node2)
        self._init_ptah()

        ptah.config.initialize_settings(
            {'group.node1': 'setting from ini'}, self.config)

        self.assertEqual(group['node1'], 'setting from ini')
        self.assertEqual(group['node2'], 10)

    def test_settings_initialize_load_settings_include(self):
        path = os.path.join(self.dir, 'settings.cfg')
        f = open(path, 'wb')
        f.write('[DEFAULT]\ngroup.node1 = value\n\n')
        f.close()

        node1 = ptah.config.SchemaNode(
                colander.Str(),
                name = 'node1',
                default = 'default1')

        node2 = ptah.config.SchemaNode(
                colander.Int(),
                name = 'node2',
                default = 10)

        group = ptah.config.register_settings('group', node1, node2)
        self._init_ptah()

        ptah.config.initialize_settings({'include': path}, self.config)

        self.assertEqual(group['node1'], 'value')
        self.assertEqual(group['node2'], 10)



from ptah import config, view
from collections import OrderedDict

from validator import All
from interfaces import _, null, required
from interfaces import Invalid, FORM_INPUT, FORM_DISPLAY

FIELD_ID = 'ptah.form:field'
PREVIEW_ID = 'ptah.form:field-preview'


def field(name, layer=''):
    info = config.DirectiveInfo(allowed_scope=('class',))

    discriminator = (FIELD_ID, name, layer)

    info.attach(
        config.ClassAction(
            view.LayerWrapper(register_field_impl, discriminator),
            (name, ),
            discriminator=discriminator)
        )


def register_field_factory(cls, name, layer=''):
    info = config.DirectiveInfo()

    discriminator = (FIELD_ID, name, layer)

    info.attach(
        config.Action(
            view.LayerWrapper(register_field_impl, discriminator),
            (cls, name),
            discriminator=discriminator)
        )


def fieldpreview(cls):
    info = config.DirectiveInfo()

    def wrapper(func):
        info.attach(
            config.Action(
                lambda config, cls, func:
                    config.get_cfg_storage(PREVIEW_ID).update({cls: func}),
                (cls, func),
                discriminator=(PREVIEW_ID, cls))
            )
        return func

    return wrapper


def register_field_impl(cfg, cls, name):
    cls.__field__ = name
    cfg.get_cfg_storage(FIELD_ID)[name] = cls


def get_field_factory(name):
    return config.get_cfg_storage(FIELD_ID).get(name, None)


def get_field_preview(cls):
    return config.get_cfg_storage(PREVIEW_ID).get(cls, None)


class Fieldset(OrderedDict):

    def __init__(self, *args, **kwargs):
        super(Fieldset, self).__init__()

        self.name = kwargs.pop('name', u'')
        self.legend = kwargs.pop('legend', u'')
        self.prefix = '%s.' % self.name if self.name else ''
        self.lprefix = len(self.prefix)

        validator = kwargs.pop('validator', None)
        if type(validator) is list:
            self.validator = All(*validator)
        else:
            self.validator = All()
            if validator is not None:
                self.validator.validators.append(validator)

        self.append(*args, **kwargs)

    def fields(self):
        for field in self.values():
            if isinstance(field, Field):
                yield field

    def fieldsets(self):
        yield self

        for fieldset in self.values():
            if isinstance(fieldset, Fieldset):
                yield fieldset

    def append(self, *args, **kwargs):
        for field in args:
            if isinstance(field, Field):
                if field.name in self:
                    raise ValueError("Duplicate name", field.name)
                self[field.name] = field

            elif isinstance(field, Fieldset):
                if field.name in self.name:
                    self.append(*field.values())
                    self.validator.validators.extend(field.validator.validators)
                    continue
                if field.name in self:
                    raise ValueError("Duplicate name", field.name)
                self[field.name] = field

            else:
                raise TypeError("Unrecognized argument type", field)

    def select(self, *names):
        return self.__class__(*[field for name, field in self.items()
                                if name in names])

    def omit(self, *names):
        return self.__class__(*[field for name, field in self.items()
                                if name not in names])

    def validate(self, data):
        self.validator(self, data)

    def bind(self, data=None, params={}):
        clone = Fieldset(
            name=self.name,
            legend=self.legend,
            prefix=self.prefix,
            validator=self.validator.validators)

        if data is None:
            data = {}

        for name, field in self.items():
            if isinstance(field, Fieldset):
                clone[name] = field.bind(
                    data.get(name, None), params)
            else:
                clone[name] = field.bind(
                    self.prefix, data.get(name, null), params)

        clone.params = params
        clone.data = data
        return clone

    def extract(self):
        data = {}
        errors = FieldsetErrors(self)

        for fieldset in self.fieldsets():
            if fieldset is self:
                continue
            fdata, ferrors = fieldset.extract()
            data[fieldset.name] = fdata
            errors.extend(ferrors)

        for field in self.fields():
            if field.mode == FORM_DISPLAY:
                continue

            value = field.missing
            try:
                form = field.extract()

                value = field.deserialize(form)
                if value is null:
                    value = field.missing

                field.validate(value)

                if field.preparer is not None:
                    value = field.preparer(value)
            except Invalid, error:
                errors.append(error)

            data[field.name[self.lprefix:]] = value

        if not errors:
            try:
                self.validate(data)
            except Invalid, error:
                errors.append(error)

        return data, errors

    def __add__(self, fieldset):
        if not isinstance(fieldset, Fieldset):
            raise ValueError(fieldset)

        return self.__class__(self, fieldset)


class FieldsetErrors(list):

    def __init__(self, fieldset, *args):
        super(FieldsetErrors, self).__init__(args)

        self.fieldset = fieldset

    @property
    def msg(self):
        r = {}
        for err in self:
            r[err.field.name] = err.msg

        return r


class Field(object):
    """Widget base class."""

    __field__ = ''

    name = ''
    title = u''
    description = u''
    required = False

    error = None
    params = {}
    mode = None
    value = null
    form_value = None

    id = None
    klass = None

    tmpl_input = None
    tmpl_display = None

    def __init__(self, name, **kw):
        self.__dict__.update(kw)

        self.name = name
        self.title = kw.get('title', name.capitalize())
        self.description = kw.get('description', u'')
        self.readonly = kw.get('readonly', None)
        self.default = kw.get('default', null)
        self.missing = kw.get('missing', required)
        self.preparer = kw.get('preparer', None)
        self.validator = kw.get('validator', None)
        self.required = self.missing is required

    def bind(self, prefix, value, params):
        clone = self.__class__.__new__(self.__class__)
        clone.__dict__.update(self.__dict__)
        clone.value = value
        clone.params = params
        clone.name = '%s%s' % (prefix, self.name)
        clone.id = clone.name.replace('.', '-')
        return clone

    def update(self, request):
        self.request = request

        if self.mode is None:
            if self.readonly:
                self.mode = FORM_DISPLAY
            else:
                self.mode = FORM_INPUT

        # extract from request
        widget_value = self.extract()
        if widget_value is not null:
            self.form_value = widget_value
            return

        value = null

        # get from value
        if self.value is not null:
            value = self.value

        # use default
        if value is null:
            value = self.default

        # Convert the value to one that the widget can understand
        if value is not null:
            value = self.serialize(value)
            self.form_value = value if value is not null else None

    def serialize(self, value):
        raise NotImplementedError()

    def deserialize(self, value):
        raise NotImplementedError()

    def validate(self, value):
        if value is required:
            raise Invalid(self, _('Required'))

        if self.validator is not None:
            self.validator(self, value)

    def extract(self, default=null):
        value = self.params.get(self.name, default)
        if value is default or not value:
            return default
        return value

    def render(self, request):
        if self.mode == FORM_DISPLAY:
            return self.tmpl_display(
                view=view,
                context=self,
                request=request)
        else:
            return self.tmpl_input(
                view=self,
                context=self,
                request=request)

    def __repr__(self):
        return '<%s %r>' % (self.__class__.__name__, self.name)


class FieldFactory(Field):

    __field__ = ''

    def __init__(self, typ, name, **kw):
        self.__field__ = typ

        super(FieldFactory, self).__init__(name, **kw)

    def bind(self, prefix, value, params):
        try:
            cls = config.get_cfg_storage(FIELD_ID)[self.__field__]
        except:
            cls = None

        if cls is None:
            raise TypeError(
                "Can't find field implementation for '%s'" % self.__field__)

        clone = cls.__new__(cls)
        clone.__dict__.update(self.__dict__)
        clone.value = value
        clone.params = params
        clone.name = '%s%s' % (prefix, self.name)
        return clone



import unittest
from pyramid.httpexceptions import HTTPForbidden


class TestFormWidgets(unittest.TestCase):
    def _makeOne(self, fields, form, request):
        from ptah.form.form import FormWidgets
        return FormWidgets(fields, form, request)

    def test_ctor(self):
        request = DummyRequest()
        form = DummyForm()
        fields = DummyFields()
        inst = self._makeOne(fields, form, request)
        self.assertEqual(inst.form_fields, fields)
        self.assertEqual(inst.form, form)
        self.assertEqual(inst.request, request)


class TestFormErrors(unittest.TestCase):

    def test_form_errors(self):
        from ptah.form import Invalid, TextField
        from ptah.form.form import FormErrorMessage
        request = DummyRequest()

        form_errors = FormErrorMessage(request)

        err1 = Invalid(None, 'Error1')
        err2 = Invalid(None, 'Error2')

        err2.field = TextField('text')

        res = form_errors.render([err1, err2])
        self.assertIn('Error1', res)
        self.assertNotIn('Error2', res)
        self.assertIn('Please fix indicated errors.', res)


class TestForm(unittest.TestCase):

    def test_basics(self):
        from ptah.form.form import Form

        request = DummyRequest()
        form = Form(None, request)

        request.url = '/test/form'
        self.assertEqual(form.action, request.url)

        self.assertEqual(form.name, 'form')

        form = Form(None, request)
        form.prefix = 'my.test.form.'
        self.assertEqual(form.name, 'my.test.form')

        self.assertEqual(form.id, 'my-test-form')

    def test_form_content(self):
        from ptah.form.form import Form

        request = DummyRequest()
        form = Form(None, request)

        self.assertIsNone(form.form_content())

        form_content = {}
        form.content = form_content
        self.assertIs(form.form_content(), form_content)

    def test_form_content_from_update(self):
        from ptah.form.form import Form

        request = DummyRequest()
        form = Form(None, request)

        form_content = {'test': 'test1'}
        form.update(**form_content)
        self.assertEqual(form.form_content(), form_content)

    def test_csrf_token(self):
        from ptah.form import form
        form.setCsrfUtility(None)

        class MyForm(form.Form):
            pass

        request = DummyRequest()
        form_ob = MyForm(None, request)

        self.assertIsNone(form.CSRF)
        self.assertIsNone(form_ob.token)

        retData = False
        class CsrfService(object):
            def generate(self, data):
                return 'csrf-token:%s'%data
            def get(self, token):
                if retData:
                    return token.split(':', 1)[-1]

        form.setCsrfUtility(CsrfService())

        token = form_ob.token
        self.assertIsNotNone(token)
        self.assertIsNone(form_ob.validate_csrf_token())

        request.POST = {}

        form_ob.csrf = True
        self.assertRaises(HTTPForbidden, form_ob.validate_csrf_token)
        self.assertRaises(HTTPForbidden, form_ob.validate, {}, [])

        request.POST = {form_ob.csrfname: token}
        self.assertRaises(HTTPForbidden, form_ob.validate_csrf_token)

        retData = True
        self.assertIsNone(form_ob.validate_csrf_token())

    def test_csrf_tokenData(self):
        from ptah.form import form

        class MyForm(form.Form):
            pass

        request = DummyRequest()
        form_ob = MyForm(None, request)

        self.assertEqual(form_ob.tokenData, 'test_form.MyForm:None')

        def authId(request):
            return 'userId'

        orig_func = form.security.authenticated_userid
        form.security.authenticated_userid = authId

        form_ob = MyForm(None, request)
        self.assertEqual(form_ob.tokenData, 'test_form.MyForm:userId')

        form.security.authenticated_userid = orig_func

    def test_form_params(self):
        from ptah.form.form import Form, DisplayForm

        request = DummyRequest()

        form = Form(None, request)
        disp_form = DisplayForm(None, request)

        self.assertEqual(form.method, 'post')

        post = {'post': 'info'}
        request.POST = post

        self.assertIs(form.form_params(), post)
        self.assertIs(disp_form.form_params(), Form.params)

        get = {'get': 'info'}
        request.GET = get
        form.method = 'get'
        self.assertIs(form.form_params(), get)

        form.method = 'unknown'
        self.assertEqual(dict(form.form_params()), {})

    def test_form_params_method(self):
        from ptah.form.form import Form

        form = Form(None, None)
        form.method = 'params'
        params = {'post': 'info'}
        form.params = params

        self.assertIs(form.form_params(), params)

    def test_form_mode(self):
        from ptah.form.form import Form, DisplayForm, \
            FORM_INPUT, FORM_DISPLAY

        request = DummyRequest()

        form = Form(None, request)
        self.assertEqual(form.mode, FORM_INPUT)

        form = DisplayForm(None, request)
        self.assertEqual(form.mode, FORM_DISPLAY)

    def test_form_update_widgets(self):
        from ptah import form

        request = DummyRequest()
        request.POST = {}

        form_ob = form.Form(None, request)
        form_ob.update()

        self.assertIsInstance(form_ob.widgets, form.FormWidgets)
        self.assertEqual(form_ob.widgets.mode, form_ob.mode)

        form_ob.mode = form.FORM_DISPLAY
        form_ob.update()
        self.assertEqual(form_ob.widgets.mode, form.FORM_DISPLAY)

        self.assertEqual(len(form_ob.widgets), 0)

        form_ob.fields = form.Fieldset(form.TextField('test'))
        form_ob.update()
        self.assertEqual(len(form_ob.widgets), 1)
        self.assertIn('test', form_ob.widgets)
        self.assertIn('test', [f.name for f in form_ob.widgets.fields()])

        self.assertIsInstance(form_ob.widgets['test'], form.TextField)
        self.assertEqual(form_ob.widgets['test'].name, 'test')
        self.assertEqual(form_ob.widgets['test'].id, 'form-widgets-test')

    def test_form_extract(self):
        from ptah import form

        request = DummyRequest()
        request.POST = {}

        form_ob = form.Form(None, request)
        form_ob.fields = form.Fieldset(form.TextField('test'))
        form_ob.update()

        data, errors = form_ob.extract()
        self.assertEqual(errors[0].msg, 'Required')

        request.POST = {'test': 'Test string'}
        form_ob.update()
        data, errors = form_ob.extract()
        self.assertEqual(data['test'], 'Test string')

    def test_form_render(self):
        from ptah import form

        request = DummyRequest()

        form_ob = form.Form(None, request)
        form_ob.fields = form.Fieldset(form.TextField('test'))
        form_ob.update()

        form_ob.render()

        def template(**data):
            return 'Form rendered'

        form_ob.template = template
        self.assertEqual(form_ob.render(), 'Form rendered')


class DummyRequest(object):
    def __init__(self):
        self.params = {}
        self.cookies = {}
        self.POST = {}


class DummyForm(object):
    prefix = 'prefix'
    def form_params(self): # pragma: no cover
        return None
    def form_content(self): # pragma: no cover
        return None


class DummyFieldset(object):
    def fieldsets(self): # pragma: no cover
        return []


class DummyFields(object):
    def __init__(self, fieldset=None):
        if fieldset is None:
            fieldset = DummyFieldset()
        self.fieldset = fieldset

    def bind(self, data, params): # pragma: no cover
        self.data = data
        self.params = params
        return self.fieldset



import colander
from pyramid.httpexceptions import HTTPForbidden

import ptah
from ptah import view, config
from ptah.authentication import authService

CONFIG_ID = 'ptah.manage:config'
MANAGE_ID = 'ptah.manage:module'
INTROSPECT_ID = 'ptah.manage:introspection'

CONFIG = config.register_settings(
    'manage',

    config.SchemaNode(
        config.Sequence(), colander.SchemaNode(colander.Str()),
        name = 'managers',
        title = 'Managers',
        description = 'List of user logins with access rights to '\
            'ptah management ui.',
        default = ()),

    config.SchemaNode(
        colander.Str(),
        name = 'manage_url',
        title = 'Ptah manage url',
        default = '/ptah-manage'),

    config.SchemaNode(
        config.Sequence(), colander.SchemaNode(colander.Str()),
        name = 'disable_modules',
        title = 'Hide Modules in Management UI',
        description = 'List of modules names to hide in manage ui',
        default = ()),

    config.SchemaNode(
        config.Sequence(), colander.SchemaNode(colander.Str()),
        name = 'disable_models',
        title = 'Hide Models in Model Management UI',
        description = 'List of models to hide in model manage ui',
        default = ('cms-type:sqlblob',)),

    title = 'Ptah manage settings',
    )


class PtahModule(object):

    #: Module name (also is used for url generation)
    name = ''

    #: Module title
    title = ''

    def __init__(self, manager, request):
        self.__parent__ = manager
        self.request = request

    def url(self):
        """ return url to this module """
        return '%s/%s'%(CONFIG.manage_url, self.name)

    @property
    def __name__(self):
        return self.name

    def available(self):
        """ is module available """
        return True


def module(id):
    info = config.DirectiveInfo(allowed_scope=('class',))

    def _complete(cfg, cls, id):
        cls.name = id
        cfg.get_cfg_storage(MANAGE_ID)[id] = cls

    info.attach(
        config.ClassAction(
            _complete, (id,),
            discriminator = (MANAGE_ID, id))
        )


def introspection(id):
    info = config.DirectiveInfo(allowed_scope=('class',))

    def _complete(cfg, cls, id):
        cls.name = id
        cfg.get_cfg_storage(INTROSPECT_ID)[id] = cls

    info.attach(
        config.ClassAction(
            _complete, (id,),
            discriminator = (INTROSPECT_ID, id))
        )


def PtahAccessManager(id):
    """ default access manager """
    managers = CONFIG.managers

    if '*' in managers:
        return True

    principal = ptah.resolve(id)

    if principal is not None and principal.login in managers:
        return True

    return False


def check_access(userid):
    return CONFIG.get('access_manager', PtahAccessManager)(userid)


def set_access_manager(func):
    CONFIG['access_manager'] = func


class PtahManageRoute(object):
    """ ptah management route """

    def __init__(self, request):
        self.request = request

        userid = authService.get_userid()
        if not check_access(userid):
            raise HTTPForbidden()

        self.manage_url = CONFIG.manage_url

    def __getitem__(self, key):
        if key not in CONFIG.disable_modules:
            mod = config.get_cfg_storage(MANAGE_ID).get(key)

            if mod is not None:
                return mod(self, self.request)

        raise KeyError(key)


view.snippettype('ptah-module-actions', PtahModule)

view.register_snippet(
    'ptah-module-actions', PtahModule,
    template = view.template('ptah.manage:templates/moduleactions.pt'))

view.register_layout(
    '', PtahManageRoute, parent='ptah-manage',
    template=view.template("ptah.manage:templates/ptah-layout.pt"))


class LayoutManage(view.Layout):
    view.layout('ptah-manage', PtahManageRoute,
                template=view.template("ptah.manage:templates/ptah-manage.pt"))

    def update(self):
        self.manage_url = CONFIG.manage_url
        self.user = authService.get_current_principal()

        mod = self.request.context
        while not isinstance(mod, PtahModule):
            mod = getattr(mod, '__parent__', None)
            if mod is None: # pragma: no cover
                break

        self.module = mod


class ManageView(view.View):
    """List ptah modules"""
    view.pview(
        context = PtahManageRoute,
        template = view.template('ptah.manage:templates/manage.pt'))

    __intr_path__ = '/ptah-manage/'

    rst_to_html = staticmethod(ptah.rst_to_html)

    def update(self):
        context = self.context
        request = self.request

        mods = []
        for name, mod in config.get_cfg_storage(MANAGE_ID).items():
            if name in CONFIG.disable_modules:
                continue
            mods.append((mod.title, mod))

        mods.sort()
        self.modules = [mod(context, request) for _t, mod in mods]



""" Base container class implementation """
import sqlalchemy as sqla
from zope import interface

import ptah
from ptah import config

from ptah.cms import events
from ptah.cms.node import Session, load_parents
from ptah.cms.content import Content
from ptah.cms.security import action
from ptah.cms.permissions import DeleteContent
from ptah.cms.interfaces import IContainer, NotFound, Error


class Container(Content):
    """ Content container implementation. """
    interface.implements(IContainer)

    _v_keys = None
    _v_keys_loaded = False
    _v_items = None

    _sql_keys = ptah.QueryFreezer(
        lambda: Session.query(Content.__name_id__)
            .filter(Content.__parent_uri__ == sqla.sql.bindparam('uri')))

    _sql_values = ptah.QueryFreezer(
        lambda: Session.query(Content)
            .filter(Content.__parent_uri__ == sqla.sql.bindparam('uri')))

    def keys(self):
        """Return an list of the keys in the container."""
        if self._v_keys_loaded:
            return self._v_keys
        else:
            if self._v_keys is None:
                self._v_keys = [n for n, in
                                self._sql_keys.all(uri=self.__uri__)]

            self._v_keys_loaded = True
            return self._v_keys

    def get(self, key, default=None):
        """Get a value for a key

        The default is returned if there is no value for the key.
        """
        if self._v_items and key in self._v_items:
            return self._v_items[key]

        item = self._sql_get_in_parent.first(key=key, parent=self.__uri__)
        if item is not None:
            item.__parent__ = self
            if not self._v_items:
                self._v_items = {key: item}
            else:
                self._v_items[key] = item
        return item

    def values(self):
        """Return an list of the values in the container."""

        if self._v_keys_loaded and self._v_items:
            if len(self._v_items) == len(self._v_keys):
                return self._v_items.values()

        values = []
        old_items = self._v_items

        self._v_keys = keys = []
        self._v_items = items = {}

        for item in self._sql_values.all(uri = self.__uri__):
            item.__parent__ = self
            items[item.__name__] = item
            keys.append(item.__name__)
            values.append(item)

        if old_items:
            for name, item in old_items.items():
                if name not in items:
                    keys.append(name)
                    items[name] = item
                    values.append(item)

        return values

    def items(self):
        """Return the items of the container."""
        for item in self.values():
            yield item.__name__, item

    def __contains__(self, key):
        """Tell if a key exists in the mapping."""
        return key in self.keys()

    def __getitem__(self, key):
        """Get a value for a key

        A KeyError is raised if there is no value for the key.
        """
        if self._v_items and key in self._v_items:
            return self._v_items[key]

        try:
            item = self._sql_get_in_parent.one(key=key, parent=self.__uri__)
            item.__parent__ = self
            if not self._v_items:
                self._v_items = {key: item}
            else:
                self._v_items[key] = item

            return item
        except sqla.orm.exc.NoResultFound:
            raise KeyError(key)

    def __setitem__(self, key, item):
        """Set a new item in the container."""

        if not isinstance(item, Content):
            raise ValueError("Content object is required")

        if item.__uri__ == self.__uri__:
            raise ValueError("Can't set to it self")

        parents = [p.__uri__ for p in load_parents(self)]
        if item.__uri__ in parents:
            raise TypeError("Can't itself to chidlren")

        if key in self.keys():
            raise KeyError(key)

        if item.__parent_uri__ is None:
            event = events.ContentAddedEvent(item)
        else:
            event = events.ContentMovedEvent(item)

        item.__name__ = key
        item.__parent__ = self
        item.__parent_uri__ = self.__uri__
        item.__path__ = '%s%s/'%(self.__path__, key)

        if item not in Session:
            Session.add(item)

        # temporary keys
        if not self._v_items:
            self._v_items = {key: item}
        else:
            self._v_items[key] = item

        if key not in self._v_keys:
            self._v_keys.append(key)

        # recursevly update children paths
        def update_path(container):
            path = container.__path__
            for item in container.values():
                item.__path__ = '%s%s/'%(path, item.__name__)

                if isinstance(item, Container):
                    update_path(item)

        if isinstance(item, Container):
            update_path(item)

        config.notify(event)

    def __delitem__(self, item, flush=True):
        """Delete a value from the container using the key."""

        if isinstance(item, basestring):
            item = self[item]

        if item.__parent_uri__ == self.__uri__:
            if isinstance(item, Container):
                for key in item.keys():
                    item.__delitem__(key, False)

            config.notify(events.ContentDeletingEvent(item))

            name = item.__name__
            if self._v_keys:
                self._v_keys.remove(name)
            if self._v_items and name in self._v_items:
                del self._v_items[name]

            if item in Session:
                try:
                    Session.delete(item)
                    if flush:
                        Session.flush()
                except:
                    pass

            return

        raise KeyError(item)

    @action
    def create(self, tname, name, **params):
        tinfo = ptah.resolve(tname)
        if tinfo is None:
            raise NotFound('Type information is not found')

        tinfo.check_context(self)

        if '/' in name:
            raise Error("Names cannot contain '/'")

        if name.startswith(' '):
            raise Error("Names cannot starts with ' '")

        if name in self:
            raise Error("Name already in use.")

        self[name] = tinfo.create()
        content = self[name]
        content.update(**params)

        return content

    @action(permission=DeleteContent)
    def batchdelete(self, uris):
        """Batch delete"""
        raise NotImplements() # pragma: no cover

    def info(self):
        info = super(Container, self).info()
        info['__container__'] = True
        return info



""" Aplication Root """
import sqlalchemy as sqla
import ptah
from ptah import config
from zope import interface

from node import Node, Session, set_policy
from container import Container
from interfaces import IApplicationRoot, IApplicationPolicy


APPFACTORY_ID = 'ptah.cms:appfactory'

def get_app_factories():
    return config.get_cfg_storage(APPFACTORY_ID)


class ApplicationRoot(Container):
    interface.implements(IApplicationRoot)

    __root_path__ = '/'

    def __resource_url__(self, request, info):
        return self.__root_path__


class ApplicationPolicy(object):
    interface.implements(IApplicationPolicy)

    __name__ = ''
    __parent__ = None

    # default acl
    __acl__ = ptah.DEFAULT_ACL

    def __init__(self, request):
        self.request = request


class ApplicationFactory(object):

    def __init__(self, factory, path='', name='', title='',
                 policy = ApplicationPolicy, default_root = None, config=None):
        self.id = '-'.join(part for part in path.split('/') if part)
        self.path = path if path.endswith('/') else '%s/'%path
        self.name = name
        self.title = title

        self.default_root = default_root
        if not path and default_root is None:
            self.default_root = True

        if isinstance(factory, type) and issubclass(factory, Node):
            factory = factory.__type__

        self.factory = factory
        self.policy = policy

        if config is not None:
            ptah.config.get_cfg_storage(
                APPFACTORY_ID, registry=config.registry)[self.id] = self

        info = ptah.config.DirectiveInfo()
        info.attach(
            ptah.config.Action(
                lambda cfg: cfg.get_cfg_storage(APPFACTORY_ID)\
                    .update({self.id:self}),
                discriminator=(APPFACTORY_ID, path))
            )

    _sql_get_root = ptah.QueryFreezer(
        lambda: Session.query(Container)\
            .filter(sqla.sql.and_(
                    Container.__name_id__ == sqla.sql.bindparam('name'),
                    Container.__type_id__ == sqla.sql.bindparam('type'))))

    def __call__(self, request=None):
        root = self._sql_get_root.first(
            name=self.name, type=self.factory.__uri__)
        if root is None:
            root = self.factory.create(title=self.title)
            root.__name_id__ = self.name
            root.__path__ = '/%s/'%root.__uri__
            Session.add(root)
            Session.flush()

        root.__root_path__ = self.path
        root.__parent__ = policy = self.policy(request)
        root.__default_root__ = self.default_root

        set_policy(policy)

        if request is not None:
            request.root = root
        return root



""" cms events """
import ptah
from ptah import config
from datetime import datetime
from zope.interface.interfaces import ObjectEvent


class ContentEvent(ObjectEvent):
    """ Base content event """

    object = None


class ContentCreatedEvent(ContentEvent):
    """ Event thrown by
        :py:class:`ptah.cms.TypeInformation` """
    config.event('Content created event')


class ContentAddedEvent(ContentEvent):
    """ Unused event.  To be removed """
    config.event('Content added event')


class ContentMovedEvent(ContentEvent):
    """ :py:class:`ptah.cms.Container` will
        notify when content has moved."""
    config.event('Content moved event')


class ContentModifiedEvent(ContentEvent):
    """ :py:class:`ptah.cms.Content` will
        notify when update() method invoked. """
    config.event('Content modified event')


class ContentDeletingEvent(ContentEvent):
    """ :py:class:`ptah.cms.Container` will
        notify when content deleted """
    config.event('Content deleting event')


@config.subscriber(ContentCreatedEvent)
def content_created_handler(ev):
    """ Assigns created, modified, __owner__
        attributes for newly created content """
    now = datetime.utcnow()
    ev.object.created = now
    ev.object.modified = now

    user = ptah.authService.get_userid()
    if user:
        ev.object.__owner__ = user


@config.subscriber(ContentModifiedEvent)
def content_modified_handler(ev):
    """ Updates the modified attribute on content """
    ev.object.modified = datetime.utcnow()



""" base class """
import unittest
import sqlahelper
import sqlalchemy
import transaction
from ptah import config
from pyramid import testing


class Base(unittest.TestCase):

    _settings = {'sqla.url': 'sqlite://',
                 'sqla.cache': False,
                 'auth.secret': 'test'}

    def _makeRequest(self, environ=None): #pragma: no cover
        from pyramid.request import Request
        if environ is None:
            environ = self._makeEnviron()
        return Request(environ)

    def _makeEnviron(self, **extras): #pragma: no cover
        environ = {
            'wsgi.url_scheme':'http',
            'wsgi.version':(1,0),
            'PATH_INFO': '/',
            'SERVER_NAME':'localhost',
            'SERVER_PORT':'8080',
            'REQUEST_METHOD':'GET',
            }
        environ.update(extras)
        return environ

    def _init_ptah(self, settings=None, handler=None, *args, **kw):
        if settings is None:
            settings = self._settings
        config.initialize(self.p_config, ('ptah', self.__class__.__module__),
                          initsettings=False)
        config.initialize_settings(settings, self.p_config)

        # create sql tables
        Base = sqlahelper.get_base()
        Base.metadata.drop_all()
        Base.metadata.create_all()
        transaction.commit()

    def _setup_pyramid(self):
        self.request = request = self._makeRequest()
        self.p_config = testing.setUp(request=request)
        self.p_config.get_routes_mapper()
        self.registry = self.p_config.registry
        self.request.registry = self.p_config.registry

    def _setRequest(self, request): #pragma: no cover
        self.request = request
        self.p_config.end()
        self.p_config.begin(request)
        request.registry = self.p_config.registry

    def setUp(self):
        try:
            engine = sqlahelper.get_engine()
        except: # pragma: no cover
            engine = sqlalchemy.engine_from_config(
                {'sqlalchemy.url': 'sqlite://'})
            sqlahelper.add_engine(engine)

        self._setup_pyramid()
        self._init_ptah()

    def tearDown(self):
        config.cleanup_system()

        sm = self.p_config
        sm.__init__('base')
        testing.tearDown()

        Session = sqlahelper.get_session()
        Session.expunge_all()



import transaction
import ptah
import sqlalchemy as sqla
from webob.multidict import MultiDict
from pyramid.testing import DummyRequest
from pyramid.httpexceptions import HTTPFound

from base import Base


class TestSqlaModuleContent(ptah.cms.Content):

    __tablename__ = 'test_sqla_content'
    __type__ = ptah.cms.Type('Test')
    name = sqla.Column(sqla.Unicode())


class TestSqlaModuleTable(ptah.cms.Base):

    __tablename__ = 'test_sqla_table'

    id = sqla.Column('id', sqla.Integer, primary_key=True)
    name = sqla.Column(sqla.Unicode())



class TestSqlaModule(Base):

    def tearDown(self):
        ptah.config.cleanup_system(self.__class__.__module__)
        super(TestSqlaModule, self).tearDown()

    def test_sqla_module(self):
        from ptah.manage.manage import CONFIG, PtahManageRoute
        from ptah.manage.sqla import SQLAModule, Table

        request = DummyRequest()

        CONFIG['managers'] = ['*']
        mr = PtahManageRoute(request)
        mod = mr['sqla']
        self.assertIsInstance(mod, SQLAModule)

        self.assertRaises(KeyError, mod.__getitem__, 'psqla-unknown')
        self.assertRaises(KeyError, mod.__getitem__, 'unknown')

        table = mod['psqla-ptah_tokens']
        self.assertIsInstance(table, Table)

    def test_sqla_traverse(self):
        from ptah.manage.sqla import SQLAModule, Table

        request = DummyRequest()

        mod = SQLAModule(None, request)

        table = mod['psqla-ptah_nodes']
        self.assertIsInstance(table, Table)

        self.assertRaises(KeyError, mod.__getitem__, 'unknown')

    def test_sqla_view(self):
        from ptah.manage.sqla import SQLAModule, MainView

        request = DummyRequest()

        mod = SQLAModule(None, request)

        res = MainView.__renderer__(mod, request)
        self.assertEqual(res.status, '200 OK')

    def test_sqla_table_view(self):
        from ptah.manage.sqla import SQLAModule, TableView

        request = DummyRequest()

        mod = SQLAModule(None, request)
        table = mod['psqla-ptah_tokens']

        res = TableView.__renderer__(table, request)
        self.assertEqual(res.status, '200 OK')
        self.assertIn('form.buttons.add', res.body)

    def test_sqla_table_view_model(self):
        from ptah.manage.sqla import SQLAModule, TableView

        ptah.cms.Session.add(TestSqlaModuleContent(title='test'))

        request = DummyRequest()

        mod = SQLAModule(None, request)
        table = mod['psqla-test_sqla_content']

        res = TableView.__renderer__(table, request).body
        self.assertIn('Inherits from:', res)
        self.assertIn('ptah_node', res)
        self.assertIn('ptah_content', res)
        self.assertNotIn('form.buttons.add', res)

    def test_sqla_table_view_model_nodes(self):
        from ptah.manage.sqla import SQLAModule, TableView

        rec = TestSqlaModuleContent(title='test')
        ptah.cms.Session.add(rec)
        ptah.cms.Session.flush()

        uri = rec.__uri__
        type_uri = rec.__type__.__uri__

        request = DummyRequest(params={'batch': 1})

        mod = SQLAModule(None, request)
        table = mod['psqla-ptah_nodes']

        res = TableView.__renderer__(table, request).body
        self.assertIn(uri, res)
        self.assertIn(type_uri, res)

        request = DummyRequest(params={'batch': 'unknown'})
        res = TableView.__renderer__(table, request).body
        self.assertIn(uri, res)

        request = DummyRequest(params={'batch': '0'})
        res = TableView.__renderer__(table, request).body
        self.assertIn(uri, res)

    def test_sqla_table_view_inheritance(self):
        from ptah.manage.sqla import SQLAModule, TableView

        request = DummyRequest()

        mod = SQLAModule(None, request)
        table = mod['psqla-ptah_tokens']

        res = TableView.__renderer__(table, request)
        self.assertEqual(res.status, '200 OK')

    def test_sqla_table_traverse(self):
        from ptah.manage.sqla import SQLAModule, Record

        from ptah import token
        from ptah.util import CSRFService

        tid = token.service.generate(CSRFService.TOKEN_TYPE, 'test')
        inst = token.service._sql_get.first(token=tid)

        request = DummyRequest()

        mod = SQLAModule(None, request)
        table = mod['psqla-ptah_tokens']

        rec = table[str(inst.id)]

        self.assertIsInstance(rec, Record)
        self.assertEqual(rec.pname, 'id')
        self.assertIsNotNone(rec.pcolumn)
        self.assertIsNotNone(rec.data)

        self.assertRaises(KeyError, table.__getitem__, 'add.html')
        self.assertRaises(KeyError, table.__getitem__, 'unknown')

    def test_sqla_table_addrec_basics(self):
        from ptah.manage.sqla import SQLAModule, AddRecord

        request = DummyRequest()

        mod = SQLAModule(None, request)
        table = mod['psqla-test_sqla_table']

        form = AddRecord(table, request)
        form.update()

        self.assertEqual(form.label, 'test_sqla_table: new record')

        request = DummyRequest(
            POST={'form.buttons.back': 'Back'})

        form = AddRecord(table, request)
        try:
            form.update()
        except Exception, res:
            pass

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], '.')

    def test_sqla_table_addrec_create(self):
        from ptah.manage.sqla import SQLAModule, AddRecord

        request = DummyRequest()

        mod = SQLAModule(None, request)
        table = mod['psqla-test_sqla_table']

        request = DummyRequest(
            session = {},
            POST={'form.buttons.create': 'Create'})

        form = AddRecord(table, request)
        form.csrf = False
        form.update()

        self.assertIn('Please fix indicated errors',
                      request.session['msgservice'][0])

        request = DummyRequest(
            session = {},
            POST={'form.buttons.create': 'Create',
                  'name': 'Test'})

        form = AddRecord(table, request)
        form.csrf = False
        try:
            form.update()
        except Exception, res:
            pass

        self.assertIn('Table record has been created.',
                      request.session['msgservice'][0])
        self.assertIsInstance(res, HTTPFound)

        rec = ptah.cms.Session.query(TestSqlaModuleTable).first()
        self.assertEqual(rec.name, 'Test')

    def test_sqla_table_addrec_create_multi(self):
        from ptah.manage.sqla import SQLAModule, AddRecord

        request = DummyRequest()

        mod = SQLAModule(None, request)
        table = mod['psqla-test_sqla_table']

        request = DummyRequest(
            session = {},
            POST={'form.buttons.createmulti': 'Create'})

        form = AddRecord(table, request)
        form.csrf = False
        form.update()

        self.assertIn('Please fix indicated errors',
                      request.session['msgservice'][0])

        request = DummyRequest(
            session = {},
            POST={'form.buttons.createmulti': 'Create',
                  'name': 'Test multi'})

        form = AddRecord(table, request)
        form.csrf = False
        form.update()

        self.assertIn('Table record has been created.',
                      request.session['msgservice'][0])

        rec = ptah.cms.Session.query(TestSqlaModuleTable).first()
        self.assertEqual(rec.name, 'Test multi')

    def test_sqla_table_editrec_basics(self):
        from ptah.manage.sqla import SQLAModule, EditRecord

        rec = TestSqlaModuleTable()
        rec.name = 'Test record'
        ptah.cms.Session.add(rec)
        ptah.cms.Session.flush()

        rec_id = rec.id

        request = DummyRequest()

        mod = SQLAModule(None, request)
        table = mod['psqla-test_sqla_table']

        rec = table[rec_id]

        form = EditRecord(rec, request)
        form.update()

        self.assertEqual(form.label, 'record 1')
        self.assertEqual(form.form_content(),
                         {'name': 'Test record'})

        request = DummyRequest(
            POST={'form.buttons.cancel': 'Cancel'})

        form = EditRecord(rec, request)
        try:
            form.update()
        except Exception, res:
            pass

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], '..')

    def test_sqla_table_editrec_modify(self):
        from ptah.manage.sqla import SQLAModule, EditRecord

        rec = TestSqlaModuleTable()
        rec.name = 'Test record'
        ptah.cms.Session.add(rec)
        ptah.cms.Session.flush()

        rec_id = rec.id

        mod = SQLAModule(None, DummyRequest())
        table = mod['psqla-test_sqla_table']

        rec = table[rec_id]

        request = DummyRequest(
            POST={'form.buttons.modify': 'Modify'})

        form = EditRecord(rec, request)
        form.csrf = False
        form.update()

        self.assertIn('Please fix indicated errors',
                      request.session['msgservice'][0])

        request = DummyRequest(
            session = {},
            POST={'form.buttons.modify': 'Modify',
                  'name': 'Record modified'})

        form = EditRecord(rec, request)
        form.csrf = False
        try:
            form.update()
        except Exception, res:
            pass

        self.assertIn('Table record has been modified.',
                      request.session['msgservice'][0])
        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], '..')

        rec = ptah.cms.Session.query(TestSqlaModuleTable).filter(
            TestSqlaModuleTable.id == rec_id).first()
        self.assertEqual(rec.name, 'Record modified')

    def test_sqla_table_editrec_remove(self):
        from ptah.manage.sqla import SQLAModule, EditRecord

        rec = TestSqlaModuleTable()
        rec.name = 'Test record'
        ptah.cms.Session.add(rec)
        ptah.cms.Session.flush()

        rec_id = rec.id

        mod = SQLAModule(None, DummyRequest())
        table = mod['psqla-test_sqla_table']

        rec = table[rec_id]

        request = DummyRequest(
            POST={'form.buttons.remove': 'Remove'})

        form = EditRecord(rec, request)
        form.csrf = False
        try:
            form.update()
        except Exception, res:
            pass

        self.assertIn('Table record has been removed.',
                      request.session['msgservice'][0])
        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], '..')

        rec = ptah.cms.Session.query(TestSqlaModuleTable).filter(
            TestSqlaModuleTable.id == rec_id).first()
        self.assertIsNone(rec, None)

    def test_sqla_table_add(self):
        from ptah.manage.sqla import SQLAModule, TableView

        mod = SQLAModule(None, DummyRequest())
        table = mod['psqla-test_sqla_table']

        request = DummyRequest(
            POST={'form.buttons.add': 'Add'})

        form = TableView(table, request)
        try:
            form.update()
        except Exception, res:
            pass

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], 'add.html')

    def test_sqla_table_remove(self):
        from ptah.manage.sqla import SQLAModule, TableView

        rec = TestSqlaModuleTable()
        rec.name = 'Test record'
        ptah.cms.Session.add(rec)
        ptah.cms.Session.flush()

        rec_id = rec.id

        mod = SQLAModule(None, DummyRequest())
        table = mod['psqla-test_sqla_table']

        request = DummyRequest(
            POST=MultiDict({'form.buttons.remove': 'Remove'}))

        form = TableView(table, request)
        form.csrf = False
        form.update()

        self.assertIn('lease select records for removing.',
                      request.session['msgservice'][0])

        request = DummyRequest(
            POST=MultiDict({'form.buttons.remove': 'Remove',
                            'rowid': 'wrong'}))

        form = TableView(table, request)
        form.csrf = False
        form.update()

        self.assertIn('lease select records for removing.',
                      request.session['msgservice'][0])

        request = DummyRequest(
            POST=MultiDict({'form.buttons.remove': 'Remove',
                            'rowid': rec_id}))

        form = TableView(table, request)
        form.csrf = False
        form.update()

        self.assertIn('Select records have been removed.',
                      request.session['msgservice'][0])

        rec = ptah.cms.Session.query(TestSqlaModuleTable).filter(
            TestSqlaModuleTable.id == rec_id).first()
        self.assertIsNone(rec, None)

    def test_sqla_table_no_remove_for_edit_model(self):
        from ptah.manage.sqla import SQLAModule, EditRecord

        rec = TestSqlaModuleContent()
        rec.name = 'Test record'
        ptah.cms.Session.add(rec)
        ptah.cms.Session.flush()

        rec_id = rec.__id__

        mod = SQLAModule(None, DummyRequest())
        table = mod['psqla-test_sqla_content']

        rec = table[rec_id]

        form = EditRecord(rec, DummyRequest())
        form.update()

        self.assertNotIn('form.buttons.remove', form.render())



""" view implementation """
import inspect
from zope import interface
from zope.interface import providedBy

from pyramid.response import Response
from pyramid.security import NO_PERMISSION_REQUIRED
from pyramid.httpexceptions import WSGIHTTPException
from pyramid.config.views import requestonly, isexception
from pyramid.interfaces import IView
from pyramid.interfaces import IRequest, IRouteRequest
from pyramid.interfaces import IViewClassifier, IExceptionViewClassifier

from ptah import config
from ptah.view.base import View
from ptah.view.customize import LayerWrapper
from ptah.view.layout import LayoutRenderer
from ptah.view.renderers import \
     PermissionRenderer, ViewRenderer, TemplateRenderer


def render_view(name, context, request):
    adapters = request.registry.adapters

    view_callable = adapters.lookup(
        (IViewClassifier, providedBy(request), providedBy(context)),
        IView, name=name, default=None)

    return view_callable(context, request)


class PyramidView(object):

    def __init__(self, chain):
        self.renderers = chain

    def __call__(self, context, request):
        content = None

        try:
            for renderer in self.renderers:
                content = renderer(context, request, content)
                if isinstance(content, Response):
                    return content
        except WSGIHTTPException, resp:
            return resp

        response = request.response
        if type(content) is unicode:
            response.unicode_body = content
        else:
            response.body = content

        return response


unset = object()

def register_view(
    name=u'', factory=View, context=None, template=None, route=None,
    layout=unset, permission=NO_PERMISSION_REQUIRED, layer=''):

    if factory is None or not callable(factory):
        raise ValueError('view factory is required')

    discriminator = ('ptah.view:view', name, context, route, layer)

    info = config.DirectiveInfo()
    info.attach(
        config.Action(
            LayerWrapper(register_view_impl, discriminator),
            (factory, name, context, template, route, layout, permission),
            discriminator = discriminator)
        )


def register_view_impl(cfg, factory, name, context, template, route_name,
                       layout, permission):
    chain = []

    # permission
    if permission != NO_PERMISSION_REQUIRED and permission is not None:
        chain.append(PermissionRenderer(permission, factory))

    # view
    if template is None:
        render = ViewRenderer(viewMapper(factory, 'render'))
    else:
        render = TemplateRenderer(viewMapper(factory, 'update'), template)
    factory.__view_renderer__ = render

    chain.append(render)

    # layout
    if layout is unset:
        layout = None
        try:
            if issubclass(factory, View):
                layout = ''
        except:
            pass

    if layout is not None:
        chain.append(LayoutRenderer(layout))

    # register view
    if isexception(context):
        view_classifier = IExceptionViewClassifier
    else:
        view_classifier = IViewClassifier

    sm = cfg.registry

    request_iface = IRequest
    if route_name is not None:
        request_iface = sm.getUtility(IRouteRequest, name=route_name)

    pview = PyramidView(chain)
    factory.__renderer__ = pview
    pview.__config_action__ = cfg.action

    sm.registerAdapter(
        pview,
        (view_classifier, request_iface, context or interface.Interface),
        IView, name)


def viewMapper(view, attr=None):
    if inspect.isclass(view):
        ronly = requestonly(view, attr)
        attr = attr or '__call__'
        meth = getattr(view, attr)
        if attr != 'update':
            update = getattr(view, 'update', None)
        else:
            update = None
        updateClass = callable(update)

        if ronly:
            def _class_request_view(context, request):
                inst = view(request)
                if updateClass:
                    update(inst)
                return inst, meth(inst)
            return _class_request_view
        else:
            def _class_view(context, request):
                inst = view(context, request)
                if updateClass:
                    update(inst)
                return inst, meth(inst)
            return _class_view
    else:
        ronly = requestonly(view)
        if ronly:
            def _request_view(context, request):
                return None, view(request)
            return _request_view
        else:
            def _view(context, request):
                return None, view(context, request)
            return _view



import unittest
from ptah import rst


class TestRST(unittest.TestCase):

    def tearDown(self):
        rst.local_data.sphinx = None

    def test_rst_py_domain(self):
        text = """ Test text :py:class:`ptah.cms.Node` """

        self.assertIn('<span class="pre">ptah.cms.Node</span>',
                      rst.rst_to_html(text))

    def test_rst_error(self):
        text = """ Test text `ptah.cms.Node` """

        self.assertEqual(
            '<pre> Test text `ptah.cms.Node` </pre>', rst.rst_to_html(text))



""" This is an example of security and URI resolver in 1 module. """
import ptah
from ptah import view
from paste.httpserver import serve

# this is login, bobdobbs whose prefix is user-example.
# the default Ptah implementation is ptah.crowd.provider, scheme `user-crowd`

USERS = {'user-example:bobdobbs':'aliens'}
SCHEME = 'user-example'

class User(object):

    def __init__(self, login):
        self.uri = '%s:%s' % (SCHEME, login)
        self.password = USERS.get(self.uri)
        self.login = login
        self.name = login

    @classmethod
    def get(cls, login):
        login = login.split(':',1)[-1]

        if USERS.get('%s:%s' % (SCHEME, login)):
            return User(login)

    get_bylogin = get_byuri = get


class UserProvider(object):

    def authenticate(self, creds):
        login, password = creds['login'], creds['password']
        user = User.get(login)
        if user is not None:
            return user

    def get_principal_bylogin(self, login):
        return User(login)


# registration
@ptah.resolver('user-example', 'An example principal resolver')
def getPrincipal(uri):
    return User.get(uri)

ptah.register_auth_provider('example', UserProvider())


if __name__ == '__main__':
    """ need to point to your settings.ini file in make_wsgi_app call.
        http://localhost:8080/show_models is url dispatch function.
        http://localhost:8080/list_children is traverser on context
        $resource_url/show_info on either folder or content.
    """
    from pyramid.config import Configurator

    config = Configurator(settings={'settings':r'./ptah.ini'})
    config.include('ptah')
    config.ptah_init()
    app = config.make_wsgi_app()

    serve(app, host='0.0.0.0')



import ptah
from pyramid.testing import DummyRequest

from base import Base


class TestPermissionsModule(Base):

    def test_perms_module(self):
        from ptah.manage.manage import CONFIG, PtahManageRoute
        from ptah.manage.permissions import PermissionsModule

        request = DummyRequest()

        ptah.authService.set_userid('test')
        CONFIG['managers'] = ('*',)
        mr = PtahManageRoute(request)
        mod = mr['permissions']

        self.assertIsInstance(mod, PermissionsModule)

    def test_perms_view(self):
        from ptah.manage.permissions import PermissionsModule, PermissionsView

        request = DummyRequest()

        mod = PermissionsModule(None, request)

        res = PermissionsView.__renderer__(mod, request)
        self.assertEqual(res.status, '200 OK')

    def test_perms_roles(self):
        from ptah.manage.permissions import PermissionsModule, RolesView

        request = DummyRequest()

        mod = PermissionsModule(None, request)

        res = RolesView.__renderer__(mod, request)
        self.assertEqual(res.status, '200 OK')



""" status message implementation """
import cgi
from zope import interface
from pyramid.interfaces import IRequest

from ptah import config
from ptah.view.tmpl import template as get_template
from ptah.view.interfaces import IMessage, IStatusMessage


def add_message(request, msg, type='info'):
    srv = IStatusMessage(request, None)
    if srv is not None:
        srv.add(msg, type)


class MessageService(object):
    """ message service """
    interface.implements(IStatusMessage)

    SESSIONKEY = 'msgservice'

    def __init__(self, request):
        self.request = request
        try:
            self.session = request.session
        except:
            self.session = {}

    def add(self, text, type='info'):
        request = self.request

        message = request.registry.getAdapter(request, IMessage, type)

        messages = self.session.get(self.SESSIONKEY, [])

        text = message.render(text)
        if text not in self.messages():
            messages.append(text)
        self.session[self.SESSIONKEY] = messages

    def clear(self):
        messages = self.session.get(self.SESSIONKEY)
        if messages is not None:
            if messages:
                del self.session[self.SESSIONKEY]
                return messages
        return ()

    def messages(self):
        messages = self.session.get(self.SESSIONKEY)
        if messages is not None:
            return messages
        return ()


class Message(object):
    interface.implements(IMessage)

    template = None

    def __init__(self, request):
        self.request = request

    def render(self, message):
        return self.template(message = message, request = self.request)


class InformationMessage(Message):
    config.adapter(IRequest, name='info')

    template = get_template('ptah.view:templates/msg-info.pt')


class SuccessMessage(Message):
    config.adapter(IRequest, name='success')

    template = get_template('ptah.view:templates/msg-success.pt')


class WarningMessage(Message):
    config.adapter(IRequest, name='warning')

    template = get_template('ptah.view:templates/msg-warning.pt')


class ErrorMessage(Message):
    config.adapter(IRequest, name='error')

    template = get_template('ptah.view:templates/msg-error.pt')

    def render(self, e):
        if isinstance(e, Exception):
            message = '%s: %s'%(e.__class__.__name__,
                                cgi.escape(str(e), True))
        else:
            message = e

        return super(ErrorMessage, self).render(message)


@config.adapter(IRequest)
@interface.implementer(IStatusMessage)
def getMessageService(request):
    service = request.registry.queryUtility(IStatusMessage)
    if service is None:
        service = MessageService(request)
    return service


def render_messages(request):
    service = IStatusMessage(request, None)
    if service is not None:
        messages = service.clear()
        if messages:
            return u'\n'.join(messages)

    return u''



""" various fields """
import datetime
from ptah import view, formatter

from interfaces import _, null, Invalid
from field import field
from fields import TextAreaField, TextField, DateTimeField


class TinymceField(TextAreaField):
    __doc__ = u'TinyMCE Text Area input widget'

    field('tinymce')

    klass = u'tinymce-widget'

    width = '400px'
    height = '300px'
    theme = "advanced"

    tmpl_input = view.template(
        "ptah.form:templates/fields/tinymce_input.pt")


class JSDateField(TextField):
    __doc__ = u'Date input widget with JQuery Datepicker.'

    field('date')

    klass = u'date-widget'
    value = u''

    tmpl_input = view.template(
        "ptah.form:templates/fields/jsdate-input.pt")

    def serialize(self, value):
        if value is null or value is None:
            return null

        if isinstance(value, datetime.datetime):
            value = value.date()

        if not isinstance(value, datetime.date):
            raise Invalid(self,
                          _('"${val}" is not a date object',
                            mapping={'val': value}))

        return value.strftime('%m/%d/%Y')

    def deserialize(self, value):
        if not value:
            return null
        try:
            return datetime.datetime.strptime(value, '%m/%d/%Y').date()
        except Exception, e:
            raise Invalid(
                self, _('Invalid date', mapping={'val': value, 'err': e}))


class JSDateTimeField(DateTimeField):
    __doc__ = u'DateTime input widget with JQuery Datepicker.'

    field('datetime')

    klass = u'datetime-widget'
    value = u''

    time_part = null
    date_part = null
    tzinfo = None

    tmpl_input = view.template(
        "ptah.form:templates/fields/jsdatetime-input.pt")

    def update(self, request):
        self.date_name = '%s.date' % self.name
        self.time_name = '%s.time' % self.name

        super(JSDateTimeField, self).update(request)

        self.date_part = self.params.get(self.date_name, null)
        self.time_part = self.params.get(self.time_name, null)

        if self.value:
            raw = self.value
            self.tzinfo = raw.tzinfo
            if self.date_part is null:
                self.date_part = raw.strftime('%m/%d/%Y')
            if self.time_part is null:
                self.time_part = raw.strftime(formatter.FORMAT.time_short)

        if self.date_part is null:
            self.date_part = u''
        if self.time_part is null:
            self.time_part = u''

    def extract(self, default=null):
        date = self.params.get(self.date_name, default)
        if date is default:
            return default

        if not date:
            return null

        time = self.params.get(self.time_name, default)
        if time is default:
            return default

        if not time:
            return null

        format = '%s %s' % ('%m/%d/%Y', formatter.FORMAT.time_short)
        try:
            dt = datetime.datetime.strptime('%s %s' % (date, time), format)
        except ValueError:
            return null

        return dt.replace(tzinfo=self.tzinfo).isoformat()



""" This is an example of useing form (imperative style). """
import ptah
from pprint import pprint
from paste.httpserver import serve
from pyramid.httpexceptions import HTTPFound
from ptah import view, form, cms


@view.pview('test-form.html', context=cms.Content)
def form_view(context, request):

    myform = form.Form(context, request)

    # define fields for form
    myform.fields = form.Fieldset(

        form.TextField(
            'title',
            title = u'Title'),  # field title

        form.TextAreaField(
            'description',
            title = u'Description',
            missing = u''), # field use this value is request doesnt contain
                            # field value, effectively field is required
                            # if `missing` is not specified
        form.TextField(
            'email',
            title = u'E-Mail',
            description = u'Please provide email address.',
            validator = form.Email(), # email validator
            ),
        )

    # form actions
    def cancelAction(form):
        raise HTTPFound(location='.')

    def updateAction(form):
        data, errors = form.extract()

        if errors:
            form.message(errors, 'form-error')
            return

        pprint(data)

        form.context.title = data['title']
        form.context.description = data['description']
        form.message('Content has been updated.', 'info')
        raise HTTPFound(location='.')

    myform.buttons.add_action('Update', action=updateAction)
    myform.buttons.add_action('Cancel', action=cancelAction)

    # form default values
    myform.content = {'title': context.title,
                      'description': context.description}

    # prepare form
    myform.update()

    # render form
    res = myform.render()

    # optional, render form in layout
    layout = view.LayoutRenderer('')
    return layout(context, request, res)


if __name__ == '__main__':
    """ ...

    """
    app = ptah.make_wsgi_app({'settings':r'./ptah.ini'})
    serve(app, host='0.0.0.0')



import colander
import sys
import unittest

from StringIO import StringIO
from pyramid.testing import setUp, tearDown
from ptah import config
from ptah.config import commands


class BaseTesting(unittest.TestCase):

    def _init_ptah(self, settings={}, *args, **kw):
        config.initialize(self.config, ('ptah.config', self.__class__.__module__))

    def setUp(self):
        self.config = setUp()
        self.registry = self.config.registry

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        tearDown()


class TestCommand(BaseTesting):

    def test_settings_command(self):
        node = config.SchemaNode(
                colander.Str(),
                name = 'node',
                default = 'test')

        group1 = config.register_settings(
            'group1', node,
            title = 'Section1',
            description = 'Description1',
            )

        group2 = config.register_settings(
            'group2', node,
            title = 'Section2',
            description = 'Description2',
            )

        self._init_ptah()

        # all
        sys.argv[1:] = ['-a']

        stdout = sys.stdout
        out = StringIO()
        sys.stdout = out

        commands.settingsCommand(False)
        sys.stdout = stdout

        val = out.getvalue()
        self.assertIn('Section1', val)
        self.assertIn('Section2', val)
        self.assertIn('group1.node', val)
        self.assertIn('group2.node', val)

        # section
        sys.argv[1:] = ['-l', 'group1']

        stdout = sys.stdout
        out = StringIO()
        sys.stdout = out

        commands.settingsCommand(False)
        sys.stdout = stdout

        val = out.getvalue()
        self.assertIn('Section1', val)
        self.assertNotIn('Section2', val)
        self.assertIn('group1.node', val)
        self.assertNotIn('group2.node', val)

        # print
        sys.argv[1:] = ['-p']

        stdout = sys.stdout
        out = StringIO()
        sys.stdout = out

        commands.settingsCommand(False)
        sys.stdout = stdout

        val = out.getvalue().strip()
        self.assertIn('group1.node = test', val)
        self.assertIn('group2.node = test', val)



""" This is an example of useing form (imperative style). """
import ptah
from pprint import pprint
from paste.httpserver import serve
from pyramid.httpexceptions import HTTPFound
from ptah import cms, view, form


class MyForm(form.Form):
    view.pview('test-form.html', context=cms.Content)

    # define fields for form
    fields = form.Fieldset(

        form.TextField(
            'title',
            title = u'Title'),  # field title

        form.TextAreaField(
            'description',
            title = u'Description',
            missing = u''), # field use this value is request doesnt contain
                            # field value, effectively field is required
                            # if `missing` is not specified
        form.TextField(
            'email',
            title = u'E-Mail',
            description = u'Please provide email address.',
            validator = form.Email(), # email validator
            ),
        )

    # form default values
    def form_content(self):
        return {'title': self.context.title,
                'description': self.context.description}

    @form.button('Update')
    def update_handler(self):
        data, errors = self.extract()

        if errors:
            self.message(errors, 'form-error')
            return

        pprint(data)

        self.context.title = data['title']
        self.context.description = data['description']
        self.message('Content has been updated.', 'info')
        raise HTTPFound(location='.')

    @form.button('Cancel')
    def cancel_handler(self):
        raise HTTPFound(location='.')


if __name__ == '__main__':
    """ ...

    """
    app = ptah.make_wsgi_app({'settings':r'./ptah.ini'})
    serve(app, host='0.0.0.0')



# ptah manage api

from ptah.manage.manage import CONFIG

from ptah.manage.manage import module
from ptah.manage.manage import introspection

from ptah.manage.manage import PtahModule
from ptah.manage.manage import PtahManageRoute

from ptah.manage.manage import check_access
from ptah.manage.manage import set_access_manager



import unittest


class TestButton(unittest.TestCase):

    def test_ctor(self):
        from ptah import form

        btn = form.Button(name='test', actionName='action',
                          actype = form.AC_PRIMARY)

        self.assertEqual(btn.name, 'test')
        self.assertEqual(btn.title, 'Test')
        self.assertEqual(btn.actype, form.AC_PRIMARY)
        self.assertEqual(repr(btn), "<Button 'test' : 'Test'>")

    def test_bind(self):
        from ptah import form

        btn = form.Button(name='test', actionName='action',
                          actype = form.AC_PRIMARY)

        params = {}
        context = object()
        request = DummyRequest()

        widget = btn.bind('test.', params, context, request)

        self.assertIsNot(btn, widget)
        self.assertIs(widget.context, context)
        self.assertIs(widget.request, request)
        self.assertIs(widget.params, params)
        self.assertEqual(widget.id, 'test-test')
        self.assertEqual(widget.name, 'test.test')
        self.assertEqual(widget.klass, 'btn primary')

    def test_activated(self):
        from ptah import form

        btn = form.Button(name='test', actionName='action',
                          actype = form.AC_PRIMARY)
        params = {'button.unkown': 'true'}
        context = object()
        request = DummyRequest()

        widget = btn.bind('button.', params, context, request)

        self.assertFalse(widget.activated())

        widget.params = {'button.test': 'true'}
        self.assertTrue(widget.activated())

    def test_render(self):
        from ptah import form

        btn = form.Button(name='test', actionName='action',
                          actype = form.AC_PRIMARY)
        params = {}
        context = object()
        request = DummyRequest()

        widget = btn.bind('test.', params, context, request)
        self.assertEqual(
            widget.render().strip(),
            """<input id="test-test" name="test.test" class="btn primary" value="Test" type="submit" />""")

    def test_execute(self):
        from ptah import form

        btn = form.Button(name='test', actionName='action')

        class Test(object):
            def action(self):
                return 'Action executed'

        self.assertRaises(AttributeError, btn, object())
        self.assertEqual(btn(Test()), 'Action executed')

        btn = form.Button(name='test', action=Test.action,
                          actype = form.AC_PRIMARY)
        self.assertEqual(btn(Test()), 'Action executed')

        btn = form.Button(name='test')
        self.assertRaises(TypeError, btn, Test())


class TestButtons(unittest.TestCase):

    def test_ctor(self):
        from ptah import form

        btn1 = form.Button(name='test1', actionName='action')
        btn2 = form.Button(name='test2', actionName='action')

        btns = form.Buttons()
        self.assertFalse(bool(btns))

        btns = form.Buttons(btn1, btn2)
        self.assertEqual(btns.keys(), [btn1.name, btn2.name])
        self.assertEqual(btns.values(), [btn1, btn2])

        btns = form.Buttons(btn1)
        self.assertEqual(btns.keys(), [btn1.name])

        btns = form.Buttons(btn2, btns)
        self.assertEqual(btns.keys(), [btn2.name, btn1.name])
        self.assertEqual(btns.values(), [btn2, btn1])

    def test_add(self):
        from ptah import form

        btn1 = form.Button(name='test1', actionName='action')
        btn2 = form.Button(name='test2', actionName='action')

        btns = form.Buttons(btn1)

        btns.add(btn2)
        self.assertEqual(btns.keys(), [btn1.name, btn2.name])
        self.assertEqual(btns.values(), [btn1, btn2])

    def test_add_duplicate(self):
        from ptah import form

        btn1 = form.Button(name='test1', actionName='action')
        btn2 = form.Button(name='test1', actionName='action')

        btns = form.Buttons(btn1)

        self.assertRaises(ValueError, btns.add, btn2)

    def test_add_action(self):
        from ptah import form

        btns = form.Buttons()

        btn1 = btns.add_action('Test action')

        self.assertIsInstance(btn1, form.Button)
        self.assertEqual(btns.keys(), [btn1.name])
        self.assertEqual(btns.values(), [btn1])

    def test_iadd(self):
        from ptah import form

        btn1 = form.Button(name='test1', actionName='action')
        btn2 = form.Button(name='test2', actionName='action')

        btns1 = form.Buttons(btn1)
        btns2 = form.Buttons(btn2)

        btns = btns1 + btns2
        self.assertEqual(btns.keys(), [btn1.name, btn2.name])
        self.assertEqual(btns.values(), [btn1, btn2])


class TestButtonDecorator(unittest.TestCase):

    def test_decorator(self):
        from ptah import form

        class MyForm(object):
            @form.button('Test button')
            def handler(self):
                """ """

        self.assertEqual(len(MyForm.buttons), 1)

        btn = MyForm.buttons.values()[0]
        self.assertEqual(btn.title, 'Test button')
        self.assertEqual(btn.actionName, 'handler')

    def test_decorator_multiple(self):
        from ptah import form

        class MyForm(object):
            @form.button('Test button')
            def handler1(self):
                """ """
            @form.button('Test button2')
            def handler2(self):
                """ """

        self.assertEqual(len(MyForm.buttons), 2)

        btn1 = MyForm.buttons.values()[0]
        btn2 = MyForm.buttons.values()[1]
        self.assertEqual(btn1.actionName, 'handler1')
        self.assertEqual(btn2.actionName, 'handler2')

    def test_create_id(self):
        from ptah.form.button import create_btn_id

        self.assertEqual(create_btn_id('Test'), 'test')
        self.assertEqual(create_btn_id('Test title'), u'Test title'.encode('hex'))


class TestActions(unittest.TestCase):

    def _makeOne(self, form, request):
        from ptah.form.button import Actions
        return Actions(form, request)

    def test_ctor(self):
        request = DummyRequest()
        form = DummyForm()
        inst = self._makeOne(form, request)
        self.assertEqual(inst.form, form)
        self.assertEqual(inst.request, request)

    def test_update(self):
        from ptah import form

        request = DummyRequest()
        tform = DummyForm()

        def disabled(form):
            return False

        res = {}
        def action1(form):
            res['action1'] = True
        def action2(form): # pragma: no cover
            res['action2'] = True

        btn1 = form.Button(name='test1', action=action1)
        btn2 = form.Button(name='test2', action=action2, condition = disabled)
        tform.buttons = form.Buttons(btn1, btn2)

        actions = self._makeOne(tform, request)
        actions.update()

        self.assertEqual(actions.keys(), [btn1.name])
        self.assertEqual(actions[btn1.name].name, 'form.buttons.test1')

        actions.execute()
        self.assertEqual(res, {})

        params = {'form.buttons.test1': 'true',
                  'form.buttons.test2': 'true'}
        tform.params = params
        actions.update()
        actions.execute()
        self.assertEqual(res, {'action1': True})


class DummyRequest(object):
    def __init__(self):
        self.params = {}
        self.cookies = {}


class DummyForm(object):
    prefix = 'form.'
    params = {}
    def __init__(self):
        self.buttons = {}
    def form_params(self):
        return self.params



""" sqlalchemy query wrapper """
import simplejson
from ptah import form
from threading import local

import sqlalchemy as sqla
from sqlalchemy import orm
from sqlalchemy.ext.mutable import Mutable
from sqlalchemy.types import TypeDecorator, VARCHAR


class QueryFreezer(object):
    """ A facade for sqla.Session.query which caches internal query structure.

    :param builder: anonymous function containing SQLAlchemy query

    .. code-block:: python

        _sql_parent = ptah.QueryFreezer(
            lambda: Session.query(Content)
                .filter(Content.__uri__ == sqla.sql.bindparam('parent')))
    """

    def __init__(self, builder):
        self.builder = builder
        self.data = local()

    def reset(self):
        self.data = local()

    def iter(self, **params):
        data = self.data
        if not hasattr(data, 'query'):
            data.query = self.builder()
            data.mapper = data.query._mapper_zero_or_none()
            data.querycontext = data.query._compile_context()
            data.querycontext.statement.use_labels = True
            data.stmt = data.querycontext.statement

        conn = data.query._connection_from_session(
            mapper=data.mapper,
            clause=data.stmt,
            close_with_result=True)

        result = conn.execute(data.stmt, **params)
        return data.query.instances(result, data.querycontext)

    def one(self, **params):
        ret = list(self.iter(**params))

        l = len(ret)
        if l == 1:
            return ret[0]
        elif l == 0:
            raise orm.exc.NoResultFound("No row was found for one()")
        else:
            raise orm.exc.MultipleResultsFound(
                "Multiple rows were found for one()")

    def first(self, **params):
        ret = list(self.iter(**params))[0:1]
        if len(ret) > 0:
            return ret[0]
        else:
            return None

    def all(self, **params):
        return list(self.iter(**params))


class JsonType(TypeDecorator):
    """Represents an immutable structure as a json-encoded string."""

    impl = VARCHAR

    def process_bind_param(self, value, dialect):
        if value is not None:
            value = simplejson.dumps(value)
        return value

    def process_result_value(self, value, dialect):
        if value is not None:
            value = simplejson.loads(value)
        return value


class MutationList(Mutable, list):

    @classmethod
    def coerce(cls, key, value):
        if not isinstance(value, MutationList):
            if isinstance(value, list):
                return MutationList(value)
            return Mutable.coerce(key, value)  # pragma: no cover
        else:
            return value

    def append(self, value):
        list.append(self, value)
        self.changed()

    def __setitem__(self, key, value):
        list.__setitem__(self, key, value)
        self.changed()

    def __delitem__(self, key):
        list.__delitem__(self, key)
        self.changed()


class MutationDict(Mutable, dict):

    @classmethod
    def coerce(cls, key, value):
        if not isinstance(value, MutationDict):
            if isinstance(value, dict):
                return MutationDict(value)
            return Mutable.coerce(key, value)  # pragma: no cover
        else:
            return value

    def __setitem__(self, key, value):
        dict.__setitem__(self, key, value)
        self.changed()

    def __delitem__(self, key):
        dict.__delitem__(self, key)
        self.changed()


def JsonDictType():
    """
    function which returns a SQLA Column Type suitable to store a Json dict.

    :returns: ptah.sqla.MutationDict
    """
    return MutationDict.as_mutable(JsonType)


def JsonListType():
    """
    function which returns a SQLA Column Type suitable to store a Json array.

    :returns: ptah.sqla.MutationList
    """

    return MutationList.as_mutable(JsonType)


def get_columns_order(mapper):
    if mapper.inherits is not None:
        order = get_columns_order(mapper.inherits)
    else:
        order = []

    table = mapper.local_table
    for cl in table.columns:
        order.append((table.name, cl.name))

    return order


def generate_fieldset(model, fieldNames=None, namesFilter=None,
                      skipPrimaryKey=True):
    """
    :param model: subclass of sqlalchemy.ext.declarative.declarative_base
    :param fieldNames: **optional** sequence of strings to use
    :param namesFilter: **optional** callable which takes a key and list
        of fieldNames to compute if fieldName should filtered out of Fieldset
        generation.
    :param skipPrimaryKey: **default: True** Should PrimaryKey be omitted
        from fieldset generation.
    :returns: a instance of :py:class:`ptah.form.Fieldset`
    """
    mapper = model.__mapper__
    order = get_columns_order(mapper)

    columns = []
    for attr in list(mapper.class_manager.attributes):
        cl = attr.__clause_element__()
        if isinstance(cl, sqla.Column):
            if fieldNames is not None and attr.key not in fieldNames:
                continue

            if namesFilter is not None and \
                    not namesFilter(attr.key, fieldNames):
                continue

            idx = order.index((cl.table.name, cl.name))
            columns.append((idx, attr.key, cl))

    columns.sort()
    columns = [(name, cl) for i, name, cl in columns]

    return build_sqla_fieldset(columns, skipPrimaryKey)


mapping = {
    (sqla.Unicode, sqla.UnicodeText, sqla.String): 'text',
    sqla.Integer: 'int',
    sqla.Float: 'float',
    sqla.Date: 'date',
    sqla.DateTime: 'datetime',
    sqla.Boolean: 'bool',
}


def build_sqla_fieldset(columns, skipPrimaryKey=False):
    """
    Given a list of SQLAlchemy columns generate a ptah.form.Fieldset.

    :param columns: sequence of sqlachemy.schema.Column instances
    :param skipPrimaryKey: **default: False** boolean whether to include PK
      Columns in Fieldset generation.
    :returns: a instance of :py:class:`ptah.form.Fieldset`
    """
    fields = []

    for name, cl in columns:
        if 'field' in cl.info:
            field = cl.info['field']
            fields.append(field)
            continue

        if cl.primary_key and skipPrimaryKey:
            continue

        typ = cl.info.get('field_type')
        if typ is None:
            for cls, field_type in mapping.items():
                if isinstance(cl.type, cls):
                    typ = field_type
                    break
        if typ is None:
            continue

        kwargs = {}
        for attr in ('missing', 'title', 'description', 'vocabulary'):
            if attr in cl.info:
                kwargs[attr] = cl.info[attr]

        if cl.primary_key and (typ == 'int'):
            kwargs['readonly'] = True

        field = form.FieldFactory(typ, name, **kwargs)
        fields.append(field)

    return form.Fieldset(*fields)



import signal
import sys
import unittest

from ptah import config


class TestShutdownHandlers(unittest.TestCase):

    def test_shutdown_handler(self):
        shutdownExecuted = []

        @config.shutdown_handler
        def shutdown():
            shutdownExecuted.append(True)

        shutdown = sys.modules['ptah.config.shutdown']
        shutdown._shutdown = False

        try:
            shutdown.processShutdown(signal.SIGINT, None)
        except BaseException, e:
            pass

        self.assertTrue(isinstance(e, KeyboardInterrupt))
        self.assertTrue(shutdownExecuted[0])

    def test_shutdown_exception_in_handler(self):
        shutdownExecuted = []

        @config.shutdown_handler
        def shutdown():
            raise ValueError()

        from ptah.config import shutdown
        shutdown._shutdown = False
        try:
            shutdown.processShutdown(signal.SIGINT, None)
        except BaseException, e:
            pass

        self.assertFalse(isinstance(e, ValueError))

    def test_shutdown_sigterm(self):
        shutdownExecuted = []

        @config.shutdown_handler
        def shutdown():
            shutdownExecuted.append(True)

        shutdown = sys.modules['ptah.config.shutdown']
        shutdown._shutdown = False
        try:
            shutdown.processShutdown(signal.SIGTERM, None)
        except:
            pass

        self.assertTrue(shutdownExecuted[0])



# ptah.config package public API

from ptah.config.api import initialize
from ptah.config.api import notify
from ptah.config.api import get_cfg_storage

from ptah.config.api import start
from ptah.config.api import Initialized
from ptah.config.api import AppStarting
from ptah.config.api import StopException

from ptah.config.api import list_packages

from ptah.config.api import cleanup
from ptah.config.api import cleanup_system

from ptah.config.directives import event
from ptah.config.directives import adapter
from ptah.config.directives import subscriber

from ptah.config.directives import Action
from ptah.config.directives import ClassAction
from ptah.config.directives import DirectiveInfo
from ptah.config.directives import ConflictError

from ptah.config.settings import register_settings
from ptah.config.settings import initialize_settings
from ptah.config.settings import SettingsInitialized
from ptah.config.settings import SettingsInitializing

from ptah.config.shutdown import shutdown
from ptah.config.shutdown import shutdown_handler

from ptah.config.schema import SchemaNode
from ptah.config.schema import Mapping
from ptah.config.schema import Sequence
from ptah.config.schema import RequiredWithDependency



import colander
from chameleon import template as chameleon_template

from ptah import config


TEMPLATE = config.register_settings(
    'template',

    config.SchemaNode(
        colander.Bool(),
        name = 'chameleon_reload',
        default = False,
        title = 'Auto reload',
        description = 'Enable chameleon templates auto reload.'),

    title = 'Templates settings',
    )


class _ViewLayersManager(object):

    def __init__(self):
        self.layers = {}

    def register(self, layer, discriminator):
        data = self.layers.setdefault(discriminator, [])
        if not layer:
            data.insert(0, layer)
        else:
            data.append(layer)

    def enabled(self, layer, discriminator):
        data = self.layers.get(discriminator)
        if data:
            return data[-1] == layer
        return False

_layersManager = _ViewLayersManager()


class LayerWrapper(object):

    def __init__(self, callable, discriminator):
        self.callable = callable
        self.layer = discriminator[-1]
        self.discriminator = discriminator[:-1]
        _layersManager.register(self.layer, self.discriminator)

    def __call__(self, cfg, *args, **kw):
        if not _layersManager.enabled(self.layer, self.discriminator):
            return # pragma: no cover

        self.callable(cfg, *args, **kw)


@config.subscriber(config.SettingsInitializing)
def initialize(*args):
    chameleon_template.AUTO_RELOAD = TEMPLATE.chameleon_reload
    chameleon_template.BaseTemplateFile.auto_reload = \
        TEMPLATE.chameleon_reload



""" interfaces """
import translationstring
from zope import interface

_ = translationstring.TranslationStringFactory('ptah')


class IURIResolver(interface.Interface):
    """ uri resolver """

    def __call__(uri):
        """ resolve uri and return context object """


class IPrincipal(interface.Interface):
    """ principal """

    uri = interface.Attribute('Unique principal id')

    name = interface.Attribute('Human readable principal name')

    login = interface.Attribute('Principal login')


class IPasswordChanger(interface.Interface):
    """ principal password changer """

    def __call__(principal, password):
        """ change password """


class IAuthInfo(interface.Interface):
    """ auth info """

    uri = interface.Attribute('Principal UUID')

    status = interface.Attribute('Status of authentication call')

    message = interface.Attribute('Failed message')

    keywords = interface.Attribute('Additional arguments from provider')

    principal = interface.Attribute('Principal object')


class IAuthentication(interface.Interface):
    """ authentication utility """

    def authenticate(credentials):
        """ authenticate credentials """

    def authenticate_principal(principal):
        """ check principal restrictions """

    def set_userid(uri):
        """ set current user """

    def get_userid():
        """ get current user """

    def get_current_principal():
        """ """

    def get_principal_bylogin(login):
        """ """


class IAuthChecker(interface.Interface):
    """ it is possible to perform additional checks on principal
    during authentication process"""

    def __call__(principal, authInfo):
        """ perform principal check """


class IAuthProvider(interface.Interface):
    """ auth provider """

    def authenticate(credentials):
        """ authenticate credentials """

    def get_principal_bylogin(login):
        """ return principal object """


class IPrincipalSearcher(interface.Interface):
    """ auth provider with search support """

    def __call__(term=''):
        """ search users return IPrincipal object """


class IOwnersAware(interface.Interface):
    """ owners aware context """

    __owner__ = interface.Attribute('Owner')


class ILocalRolesAware(interface.Interface):
    """ local roles aware context """

    __local_roles__ = interface.Attribute('Local roles dict')


class IACLsAware(interface.Interface):
    """ acl maps aware context """

    __acls__ = interface.Attribute('List of acl map ids')


class IIntrospection(interface.Interface):
    """ introspection handler """

    name = interface.Attribute('Name')
    title = interface.Attribute('Title')

    def __init__(request):
        """ constructor """

    def renderAction(action):
        """ render config actions """

    def renderActions(*actions):
        """ render config actions """



""" commands tests """
import sys, os
import unittest
import tempfile, shutil
from cStringIO import StringIO
from paste.script.command import run
from ptah import config, view


class TestStaticCommand(unittest.TestCase):

    def setUp(self):
        self.out = StringIO()
        self._stdout = sys.stdout
        sys.stdout = self.out
        self.dir = tempfile.mkdtemp()

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)

        self.stdout = self._stdout
        shutil.rmtree(self.dir)

    def test_commands_static_list(self):
        view.static('tests', 'ptah.view.tests:static')

        _out = sys.stdout

        sys.argv[:] = ['paste', 'static', '-l']

        out = StringIO()
        sys.stdout = out

        try:
            print run()
        except SystemExit:
            pass

        sys.stdout = _out

        val = out.getvalue()
        self.assertTrue('* tests' in val)
        self.assertTrue('by: ptah.view.tests' in val)

    def test_commands_static_dump_errors(self):
        from ptah.view.commands import StaticCommand
        StaticCommand._include = ('ptah', self.__class__.__module__)

        view.static('tests', 'ptah.view.tests:static/dir2')

        _out = sys.stdout

        file = os.path.join(self.dir, 'file')
        f = open(file, 'w')
        f.write(' ')
        f.close()

        sys.argv[:] = ['paste', 'static', '-d %s'%file]
        out = StringIO()
        sys.stdout = out

        try:
            run()
        except SystemExit:
            pass

        sys.stdout = _out
        val = out.getvalue()
        self.assertTrue('Output path is not directory.' in val)

    def test_commands_static_dump(self):
        view.static('tests', 'ptah.view.tests:static/dir2')

        from ptah.view.commands import StaticCommand
        StaticCommand._include = ('ptah', self.__class__.__module__)

        dir = os.path.join(self.dir, 'subdir')

        sys.argv[:] = ['paste', 'static', '-d %s'%dir]
        try:
            run()
        except SystemExit:
            pass

        val = self.out.getvalue()
        self.assertTrue("* Coping from 'ptah.view.tests'" in val)
        files = os.listdir(os.path.join(dir, 'tests'))
        files.sort()
        self.assertTrue(files == ['style.css', 'text.txt'])

    def test_commands_static_dump_skipping_existing(self):
        from ptah.view.commands import StaticCommand
        StaticCommand._include = ('ptah', self.__class__.__module__)

        view.static('tests', 'ptah.view.tests:static/dir2')

        os.makedirs(os.path.join(self.dir, 'tests'))
        file = os.path.join(self.dir, 'tests', 'text.txt')
        f = open(file, 'wb')
        f.write('test existing file')
        f.close()

        sys.argv[:] = ['paste', 'static', '-d %s'%self.dir]
        try:
            run()
        except SystemExit:
            pass

        val = self.out.getvalue()
        self.assertTrue("skipping ../ptah.view.tests/text.txt" in val)
        self.assertTrue('test existing file' == open(file, 'r').read())



import sqlalchemy as sqla
import transaction
import ptah
from ptah import cms
from webob.multidict import MultiDict
from pyramid.testing import DummyRequest
from pyramid.httpexceptions import HTTPFound

from base import Base


class Content1(cms.Content):
    __type__ = cms.Type('content1')


class Content2(cms.Node):
    __type__ = cms.Type('content2')

    title = sqla.Column(sqla.Unicode, default=u'')


class TestModelModule(Base):

    def test_model_module(self):
        from ptah.manage.model import ModelModule
        from ptah.manage.manage import CONFIG, PtahManageRoute

        request = DummyRequest()

        ptah.authService.set_userid('test')
        CONFIG['managers'] = ('*',)
        mr = PtahManageRoute(request)
        mod = mr['models']

        self.assertIsInstance(mod, ModelModule)

    def test_model_view(self):
        from ptah.manage.model import ModelModule, ModelModuleView

        request = DummyRequest()

        mod = ModelModule(None, request)

        res = ModelModuleView.__renderer__(mod, request)
        self.assertEqual(res.status, '200 OK')
        self.assertIn('content1', res.body)
        self.assertIn('content2', res.body)

    def test_model_view_disabled(self):
        from ptah.manage.manage import CONFIG
        from ptah.manage.model import ModelModule, ModelModuleView

        CONFIG['disable_models'] = ['cms-type:content2']

        request = DummyRequest()

        mod = ModelModule(None, request)

        res = ModelModuleView.__renderer__(mod, request)
        self.assertEqual(res.status, '200 OK')
        self.assertIn('content1', res.body)
        self.assertNotIn('content2', res.body)

    def test_model_traverse(self):
        from ptah.manage.model import ModelModule, Model

        request = DummyRequest()

        mod = ModelModule(None, request)

        item = mod['content1']
        self.assertIsInstance(item, Model)
        self.assertIs(item.tinfo, Content1.__type__)

        item = mod['content2']
        self.assertIsInstance(item, Model)
        self.assertIs(item.tinfo, Content2.__type__)

        self.assertRaises(KeyError, mod.__getitem__, 'unknown')


class TestModel(Base):

    def test_model_traverse(self):
        from ptah.manage.model import ModelModule, Record

        content = Content1()
        content.title = u'Content test'

        ptah.cms.Session.add(content)
        ptah.cms.Session.flush()

        rowid = content.__id__
        transaction.commit()

        mod = ModelModule(None, DummyRequest())
        model = mod['content1']

        rec = model[str(rowid)]
        self.assertIsNotNone(rec)
        self.assertIsInstance(rec, Record)
        self.assertIsInstance(rec.record, Content1)
        self.assertEqual(rec.record.__id__, rowid)
        self.assertRaises(KeyError, model.__getitem__, 'add.html')
        self.assertRaises(KeyError, model.__getitem__, 'unknown')

    def test_model_remove_errors(self):
        from ptah.manage.model import ModelModule, ModelView

        mod = ModelModule(None, DummyRequest())
        model = mod['content1']

        form = ModelView(
            model, DummyRequest(
                POST=MultiDict({'form.buttons.remove': 'Remove'})))
        form.csrf = False
        form.update()

        self.assertIn("Please select records for removing.",
                      form.request.session['msgservice'][0])

    def test_model_remove(self):
        from ptah.manage.model import ModelModule, ModelView

        content = Content1()
        content.title = u'Content test'

        ptah.cms.Session.add(content)
        ptah.cms.Session.flush()

        rowid = content.__id__
        transaction.commit()

        mod = ModelModule(None, DummyRequest())
        model = mod['content1']

        form = ModelView(
            model, DummyRequest(
                POST=MultiDict({'rowid':rowid,
                                'form.buttons.remove': 'Remove'})))
        form.csrf = False
        try:
            form.update()
        except Exception, res:
            pass

        self.assertIsInstance(res, HTTPFound)

        transaction.commit()

        rec = ptah.cms.Session.query(Content1).filter(
            Content1.__id__ == rowid).first()
        self.assertIsNone(rec)

    def test_model_list(self):
        from ptah.manage.model import ModelModule, ModelView

        content = Content1()
        content.title = u'Content test'

        ptah.cms.Session.add(content)
        ptah.cms.Session.flush()

        rowid = content.__id__
        transaction.commit()

        mod = ModelModule(None, DummyRequest())
        model = mod['content1']

        res = ModelView.__renderer__(model, DummyRequest())
        self.assertIn('value="%s"'%rowid, res.body)

        res = ModelView.__renderer__(model, DummyRequest(
                params={'batch': 0}))
        self.assertIn('value="%s"'%rowid, res.body)

        res = ModelView.__renderer__(model, DummyRequest(
                params={'batch': 'unknown'}))
        self.assertIn('value="%s"'%rowid, res.body)

    def test_model_add(self):
        from ptah.manage.model import ModelModule, ModelView

        mod = ModelModule(None, DummyRequest())
        model = mod['content1']

        form = ModelView(
            model, DummyRequest(
                POST=MultiDict({'form.buttons.add': 'Add'})))
        form.csrf = False
        try:
            form.update()
        except Exception, res:
            pass

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], 'add.html')


class TestAddRecord(Base):

    def test_model_add_errors(self):
        from ptah.manage.model import ModelModule, AddRecord

        mod = ModelModule(None, DummyRequest())
        model = mod['content1']

        form = AddRecord(
            model, DummyRequest(
                POST=MultiDict({'form.buttons.add': 'Add'})))
        form.csrf = False
        form.update()

        self.assertIn("Please fix indicated errors.",
                      form.request.session['msgservice'][0])

    def test_model_add_back(self):
        from ptah.manage.model import ModelModule, AddRecord

        mod = ModelModule(None, DummyRequest())
        model = mod['content1']

        form = AddRecord(
            model, DummyRequest(
                POST=MultiDict({'form.buttons.back': 'Back'})))
        form.csrf = False
        try:
            form.update()
        except Exception, res:
            pass

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], '.')

    def test_model_add(self):
        from ptah.manage.model import ModelModule, AddRecord

        mod = ModelModule(None, DummyRequest())
        model = mod['content1']

        form = AddRecord(
            model, DummyRequest(
                POST=MultiDict({'title': u'Test content',
                                'form.buttons.add': 'Add'})))
        form.csrf = False
        try:
            form.update()
        except Exception, res:
            pass

        id = form.record.__id__
        transaction.commit()

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], './%s/'%id)
        self.assertEqual(len(ptah.cms.Session.query(Content1).all()), 1)

        content = ptah.cms.Session.query(Content1) \
            .filter(Content1.__id__ == id).first()
        self.assertEqual(content.title, u'Test content')

    def test_model_add_node(self):
        from ptah.manage.model import ModelModule, AddRecord

        mod = ModelModule(None, DummyRequest())
        model = mod['content2']

        form = AddRecord(
            model, DummyRequest(
                POST=MultiDict({'title': u'Test content',
                                'form.buttons.add': 'Add'})))
        form.csrf = False
        try:
            form.update()
        except Exception, res:
            pass

        id = form.record.__id__
        transaction.commit()

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], './%s/'%id)
        self.assertEqual(len(ptah.cms.Session.query(Content2).all()), 1)

        content = ptah.cms.Session.query(Content2) \
            .filter(Content2.__id__ == id).first()
        self.assertEqual(content.title, u'Test content')


class TestEditRecord(Base):

    def test_model_edit_back(self):
        from ptah.manage.model import ModelModule, EditRecord

        content = Content1()
        content.title = u'Content test'

        ptah.cms.Session.add(content)
        ptah.cms.Session.flush()

        rowid = content.__id__
        transaction.commit()

        mod = ModelModule(None, DummyRequest())
        model = mod['content1'][rowid]

        form = EditRecord(
            model, DummyRequest(
                POST=MultiDict({'form.buttons.back': 'Back'})))
        form.csrf = False
        try:
            form.update()
        except Exception, res:
            pass

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], '../')

    def test_model_edit_errors(self):
        from ptah.manage.model import ModelModule, EditRecord

        content = Content1()
        content.title = u'Content test'

        ptah.cms.Session.add(content)
        ptah.cms.Session.flush()

        rowid = content.__id__
        transaction.commit()

        mod = ModelModule(None, DummyRequest())
        model = mod['content1'][rowid]

        form = EditRecord(
            model, DummyRequest(
                POST=MultiDict({'form.buttons.modify': 'Modify'})))
        form.csrf = False
        form.update()

        self.assertIn("Please fix indicated errors.",
                      form.request.session['msgservice'][0])

    def test_model_edit(self):
        from ptah.manage.model import ModelModule, EditRecord

        content = Content1()
        content.title = u'Content test'

        ptah.cms.Session.add(content)
        ptah.cms.Session.flush()

        rowid = content.__id__
        transaction.commit()

        mod = ModelModule(None, DummyRequest())
        model = mod['content1'][rowid]

        form = EditRecord(
            model, DummyRequest(
                POST=MultiDict({'title': 'Content',
                                'form.buttons.modify': 'Modify'})))
        form.csrf = False
        form.update()

        self.assertEqual(form.label, 'Record id: %s'%rowid)

        self.assertIn("Model record has been modified.",
                      form.request.session['msgservice'][0])
        transaction.commit()

        content = ptah.cms.Session.query(Content1) \
            .filter(Content1.__id__ == rowid).first()
        self.assertEqual(content.title, u'Content')

    def test_model_edit_node(self):
        from ptah.manage.model import ModelModule, EditRecord

        content = Content2()
        content.title = u'Content test'

        ptah.cms.Session.add(content)
        ptah.cms.Session.flush()

        rowid = content.__id__
        transaction.commit()

        mod = ModelModule(None, DummyRequest())
        model = mod['content2'][rowid]

        form = EditRecord(
            model, DummyRequest(
                POST=MultiDict({'title': 'Content',
                                'form.buttons.modify': 'Modify'})))
        form.csrf = False
        form.update()

        self.assertIn("Model record has been modified.",
                      form.request.session['msgservice'][0])
        transaction.commit()

        content = ptah.cms.Session.query(Content2) \
            .filter(Content2.__id__ == rowid).first()
        self.assertEqual(content.title, u'Content')


class TestTypeIntrospect(Base):

    def test_type_introspect(self):
        from ptah.config import directives
        from ptah.manage.model import TypeIntrospection

        data = directives.scan(self.__class__.__module__, set())

        actions = []
        for action in data:
            if action.discriminator[0] == 'ptah-cms:type':
                actions.append(action)

        ti = TypeIntrospection(self.request)
        res = ti.renderActions(*actions)

        self.assertIn('<small>cms-type:content1</small>', res)
        self.assertIn('<small>cms-type:content2</small>', res)



# ptah.form public api

from ptah.form.interfaces import null
from ptah.form.interfaces import required
from ptah.form.interfaces import Invalid

# field
from ptah.form.field import Field
from ptah.form.field import FieldFactory
from ptah.form.field import Fieldset
from ptah.form.field import FieldsetErrors

# field registration
from ptah.form.field import field
from ptah.form.field import fieldpreview
from ptah.form.field import get_field_factory
from ptah.form.field import get_field_preview
from ptah.form.field import register_field_factory

# vocabulary
from ptah.form.vocabulary import SimpleTerm
from ptah.form.vocabulary import SimpleVocabulary

# widget mode
from ptah.form.interfaces import FORM_INPUT
from ptah.form.interfaces import FORM_DISPLAY

# validators
from ptah.form.validator import All
from ptah.form.validator import Function
from ptah.form.validator import Regex
from ptah.form.validator import Email
from ptah.form.validator import Range
from ptah.form.validator import Length
from ptah.form.validator import OneOf

# fields
from ptah.form.fields import TextField
from ptah.form.fields import IntegerField
from ptah.form.fields import FloatField
from ptah.form.fields import DecimalField
from ptah.form.fields import TextAreaField
from ptah.form.fields import FileField
from ptah.form.fields import LinesField
from ptah.form.fields import PasswordField
from ptah.form.fields import DateField
from ptah.form.fields import DateTimeField
from ptah.form.fields import RadioField
from ptah.form.fields import BoolField
from ptah.form.fields import ChoiceField
from ptah.form.fields import MultiChoiceField
from ptah.form.fields import MultiSelectField

# js fields (temporary)
from ptah.form.jsfields import JSDateField
from ptah.form.jsfields import JSDateTimeField
from ptah.form.jsfields import TinymceField

# helper field classes
from ptah.form.fields import VocabularyField
from ptah.form.fields import BaseChoiceField
from ptah.form.fields import BaseMultiChoiceField

# forms
from ptah.form.form import Form
from ptah.form.form import DisplayForm
from ptah.form.form import FormWidgets
from ptah.form.form import setCsrfUtility

# form snippets
from ptah.form.form import FORM_VIEW
from ptah.form.form import FORM_ACTIONS
from ptah.form.form import FORM_WIDGET
from ptah.form.form import FORM_DISPLAY_WIDGET

# button
from ptah.form.button import button
from ptah.form.button import Button
from ptah.form.button import Buttons
from ptah.form.button import AC_DEFAULT
from ptah.form.button import AC_PRIMARY
from ptah.form.button import AC_DANGER
from ptah.form.button import AC_SUCCESS
from ptah.form.button import AC_INFO



import unittest
import ptah
from ptah import form, config

from base import Base


class Principal(object):

    def __init__(self, uri, name, login):
        self.uri = uri
        self.name = name
        self.login = login


class TestPasswordSchema(Base):

    def test_password_required(self):
        self._init_ptah()
        from ptah.password import PasswordSchema

        pwdSchema = PasswordSchema.bind(params={})

        data, errors = pwdSchema.extract()
        self.assertEqual(len(errors), 2)
        self.assertEqual(errors[0].field.name, 'password')

    def test_password_equal(self):
        self._init_ptah()
        from ptah.password import PasswordSchema

        pwdSchema = PasswordSchema.bind(
            params={'password': '12345', 'confirm_password': '123456'})
        data, errors = pwdSchema.extract()

        self.assertEqual(len(errors), 1)
        self.assertIsInstance(errors[0].field, form.Fieldset)
        self.assertEqual(
            errors[0].msg[0],
            "Password and Confirm Password should be the same.")

    def test_password(self):
        self._init_ptah()
        from ptah.password import PasswordSchema

        pwdSchema = PasswordSchema.bind(
            params={'password': '12345', 'confirm_password': '12345'})
        data, errors = pwdSchema.extract()
        self.assertEqual(len(errors), 0)

    def test_password_validator(self):
        from ptah.password import passwordValidator, PasswordTool

        vp = PasswordTool.validate

        def validatePassword(self, pwd):
            return 'Error'

        PasswordTool.validate = validatePassword

        self.assertRaises(
            form.Invalid, passwordValidator, None, 'pwd')

        PasswordTool.validate = vp


class TestSHAPasswordManager(unittest.TestCase):

    def test_password_ssha(self):
        from ptah.password import SSHAPasswordManager

        manager = SSHAPasswordManager()

        password = u"right \N{CYRILLIC CAPITAL LETTER A}"
        encoded = manager.encode(password, salt="")

        self.assertEqual(encoded, '{ssha}BLTuxxVMXzouxtKVb7gLgNxzdAI=')
        self.assertTrue(manager.check(encoded, password))
        self.assertFalse(manager.check(encoded, password + u"wrong"))

        encoded = manager.encode(password)
        self.assertTrue(manager.check(encoded, password))


class TestPlainPasswordManager(unittest.TestCase):

    def test_password_plain(self):
        from ptah.password import PlainPasswordManager

        manager = PlainPasswordManager()

        password = u"test pwd"
        encoded = manager.encode(password, salt="")

        self.assertEqual(encoded, '{plain}test pwd')
        self.assertTrue(manager.check(encoded, password))
        self.assertTrue(manager.check(password, password))
        self.assertFalse(manager.check(encoded, password + u"wrong"))

        encoded = manager.encode(password)
        self.assertTrue(manager.check(encoded, password))


class TestPasswordSettings(Base):

    def test_password_settings(self):
        self._init_ptah()

        from ptah.password import \
             PlainPasswordManager, SSHAPasswordManager

        ptah.PWD_CONFIG['manager'] = 'unknown'

        self.assertIsInstance(ptah.pwd_tool.manager, PlainPasswordManager)

        ptah.PWD_CONFIG['manager'] = 'ssha'

        self.assertIsInstance(ptah.pwd_tool.manager, SSHAPasswordManager)


class TestPasswordChangerDecl(Base):

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        super(TestPasswordChangerDecl, self).tearDown()

    def test_password_changer_decl(self):
        import ptah

        @ptah.password_changer('test-schema')
        def changer(schema):
            """ """

        self._init_ptah()

        from ptah.password import PASSWORD_CHANGER_ID
        changers = config.get_cfg_storage(PASSWORD_CHANGER_ID)

        self.assertIn('test-schema', changers)
        self.assertIs(changers['test-schema'], changer)

    def test_password_changer_decl_conflict(self):
        import ptah

        @ptah.password_changer('test-schema')
        def changer(schema):
            """ """

        @ptah.password_changer('test-schema')
        def changer2(schema):
            """ """

        self.assertRaises(config.ConflictError, self._init_ptah)

    def test_password_changer(self):
        import ptah

        @ptah.password_changer('test-schema')
        def changer(schema):
            """ """

        self._init_ptah()

        p = Principal('test-schema:numbers_numbers', 'name', 'login')
        self.assertTrue(ptah.pwd_tool.can_change_password(p))

        p = Principal('unknown-schema:numbers_numbers', 'name', 'login')
        self.assertFalse(ptah.pwd_tool.can_change_password(p))


class TestPasswordTool(Base):

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        super(TestPasswordTool, self).tearDown()

    def test_password_encode(self):
        ptah.PWD_CONFIG['manager'] = 'plain'

        encoded = ptah.pwd_tool.encode('12345')
        self.assertEqual(encoded, '{plain}12345')

    def test_password_check(self):
        ptah.PWD_CONFIG['manager'] = 'ssha'

        self.assertFalse(ptah.pwd_tool.check('12345', '12345'))
        self.assertTrue(ptah.pwd_tool.check('{plain}12345', '12345'))
        self.assertFalse(ptah.pwd_tool.check('{plain}12345', '123455'))
        self.assertFalse(ptah.pwd_tool.check('{unknown}12345', '123455'))

    def test_password_passcode(self):
        p = Principal('test-schema:test', 'name', 'login')
        principals = {'test-schema:test': p}

        @ptah.resolver('test-schema')
        def resolver(uri):
            return principals.get(uri)

        self._init_ptah()

        token = ptah.pwd_tool.generate_passcode(p)
        self.assertIsNotNone(token)

        newp = ptah.pwd_tool.get_principal(token)
        self.assertIs(newp, p)

        ptah.pwd_tool.remove_passcode(token)

        newp = ptah.pwd_tool.get_principal(token)
        self.assertIsNone(newp)

    def test_password_change_password(self):
        p = Principal('test-schema:test', 'name', 'login')
        principals = {'test-schema:test': p}

        @ptah.resolver('test-schema')
        def resolver(uri):
            return principals.get(uri)

        @ptah.password_changer('test-schema')
        def changer(principal, password):
            p.password = password

        self._init_ptah()

        token = ptah.pwd_tool.generate_passcode(p)
        self.assertIsNotNone(token)

        newp = ptah.pwd_tool.get_principal(token)
        self.assertIs(newp, p)

        self.assertTrue(ptah.pwd_tool.change_password(token, '12345'))
        self.assertEqual(p.password, '{plain}12345')

        newp = ptah.pwd_tool.get_principal(token)
        self.assertIsNone(newp)

        self.assertFalse(ptah.pwd_tool.change_password('unknown', '12345'))

    def test_password_validate(self):
        ptah.PWD_CONFIG['min_length'] = 5
        self.assertEqual(ptah.pwd_tool.validate('1234'),
                         'Password should be at least 5 characters.')

        ptah.PWD_CONFIG['letters_digits'] = True
        self.assertEqual(ptah.pwd_tool.validate('123456'),
                         'Password should contain both letters and digits.')

        ptah.PWD_CONFIG['letters_mixed_case'] = True
        self.assertEqual(ptah.pwd_tool.validate('abs456'),
                         'Password should contain letters in mixed case.')

        self.assertIsNone(ptah.pwd_tool.validate('aBs456'))



""" type implementation """
import ptah, sys, logging
import sqlalchemy as sqla
from ptah import config
from zope import interface

from node import Session
from content import Content
from container import Container
from security import build_class_actions
from events import ContentCreatedEvent
from interfaces import Forbidden, ContentSchema, ITypeInformation

log = logging.getLogger('ptah.cms')

TYPES_DIR_ID = 'ptah-cms:type'

@ptah.resolver('cms-type')
def typeInfoResolver(uri):
    """Type resolver

       :Parameters:
         - type scheme, e.g. blob-sql
       :Returns:
         - :py:class:`ptah.cms.TypeInformation`
    """
    return config.get_cfg_storage(TYPES_DIR_ID).get(uri)


def get_type(uri):
    """
    :param uri: string identifier for TypeInformation, e.g. `cms-type:sqlblob`

    :Returns:
      - :py:class:`ptah.cms.TypeInformation`

    """
    return config.get_cfg_storage(TYPES_DIR_ID).get(uri)


def get_types():
    """
    :Returns:
      - mapping of all registered identifier and TypeInformation
    """
    return config.get_cfg_storage(TYPES_DIR_ID)


class TypeInformation(object):
    interface.implements(ITypeInformation)

    fieldset = None
    description = u''
    permission = ptah.NOT_ALLOWED

    addview = None # addview action, path relative to current container
    filter_content_types = False
    allowed_content_types = ()
    global_allow = True

    def __init__(self, cls, name, title, fieldset, **kw):
        self.__dict__.update(kw)

        self.__uri__ = 'cms-type:%s'%name

        self.cls = cls
        self.name = name
        self.title = title
        self.fieldset = fieldset

    def create(self, **data):
        content = self.cls(**data)
        config.notify(ContentCreatedEvent(content))
        return content

    def is_allowed(self, container):
        if not isinstance(container, Container):
            return False

        if self.permission:
            return ptah.check_permission(self.permission, container)
        return True

    def check_context(self, container):
        if not self.is_allowed(container):
            raise Forbidden()

    def list_types(self, container):
        if container.__type__ is not self or \
               not isinstance(container, Container):
            return ()

        types = []
        all_types = config.get_cfg_storage(TYPES_DIR_ID)

        if self.filter_content_types:
            for tinfo in self.allowed_content_types:
                if isinstance(tinfo, basestring):
                    tinfo = all_types.get('cms-type:%s'%tinfo)

                if tinfo and tinfo.is_allowed(container):
                    types.append(tinfo)
        else:
            for tinfo in all_types.values():
                if tinfo.global_allow and tinfo.is_allowed(container):
                    types.append(tinfo)

        return types


def Type(name, title=None, fieldset=None, **kw):
    """ Declare new type. This function has to be called within a content
    class declaration.

    .. code-block:: python

        class MyContent(ptah.cms.Content):

            __type__ = Type('My content')

    """
    info = config.DirectiveInfo(allowed_scope=('class',))

    fs = ContentSchema if fieldset is None else fieldset

    if title is None:
        title = name.capitalize()

    typeinfo = TypeInformation(None, name, title, fs, **kw)

    f_locals = sys._getframe(1).f_locals
    if '__mapper_args__' not in f_locals:
        f_locals['__mapper_args__'] = {'polymorphic_identity': typeinfo.__uri__}
    if '__id__' not in f_locals and '__tablename__' in f_locals:
        f_locals['__id__'] = sqla.Column(
            'id', sqla.Integer,
            sqla.ForeignKey('ptah_content.id'), primary_key=True)
    if '__uri_factory__' not in f_locals:
        f_locals['__uri_factory__'] = ptah.UriFactory('cms-%s'%name)

        def resolve_content(uri):
            return typeinfo.cls.__uri_sql_get__.first(uri=uri)

        resolve_content.__doc__ = 'CMS Content resolver for %s type'%title

        ptah.register_uri_resolver('cms-%s'%name, resolve_content, depth = 2)

    info.attach(
        config.ClassAction(
            register_type_impl, (typeinfo, name, fieldset), kw,
            discriminator = (TYPES_DIR_ID, name))
        )

    return typeinfo


excludeNames = ('expires', 'contributors', 'creators', 'view', 'subjects',
                'publisher', 'effective', 'created', 'modified')
def names_filter(name, fieldNames=None):
    if fieldNames is not None and name in fieldNames:
        return True

    if name in excludeNames:
        return False

    return not name.startswith('_')


def register_type_impl(
    config, cls, tinfo, name, fieldset,
    permission = ptah.NOT_ALLOWED, fieldNames=None, **kw):

    # generate schema
    if fieldset is None:
        fieldset = ptah.generate_fieldset(
            cls, fieldNames=fieldNames, namesFilter=names_filter)
        log.info("Generating fieldset for %s content type.", cls)

    if 'global_allow' not in kw and not issubclass(cls, Content):
        kw['global_allow'] = False

    tinfo.__dict__.update(kw)

    if fieldset is not None:
        tinfo.fieldset = fieldset

    tinfo.cls = cls
    tinfo.permission = permission

    config.get_cfg_storage(TYPES_DIR_ID)[tinfo.__uri__] = tinfo

    # sql query for content resolver
    cls.__uri_sql_get__ = ptah.QueryFreezer(
        lambda: Session.query(cls)
        .filter(cls.__uri__ == sqla.sql.bindparam('uri')))

    # build cms actions
    build_class_actions(cls)



""" uri resolver """
import uuid
from ptah import config

RESOLVER_ID = 'ptah:uri-resolver'


def resolve(uri):
    """ Resolve uri, return resolved object.

    Uri contains two parts, `schema` and `uuid`. `schema` is used for
    resolver selection. `uuid` is resolver specific data. By default
    uuid is a uuid.uuid4 string.
    """
    if not uri:
        return

    try:
        schema, data = uri.split(':', 1)
    except ValueError:
        return None

    try:
        return config.get_cfg_storage(RESOLVER_ID)[schema](uri)
    except KeyError:
        pass

    return None


def extract_uri_schema(uri):
    """ Extract schema of given uri """
    if uri:
        try:
            schema, data = uri.split(':', 1)
            return schema
        except:
            pass

    return None


def resolver(schema):
    """ Register resolver for given schema
    """
    info = config.DirectiveInfo()

    def wrapper(func):
        info.attach(
            config.Action(
                _register_uri_resolver, (schema, func),
                discriminator=(RESOLVER_ID, schema))
            )

        return func

    return wrapper


def register_uri_resolver(schema, resolver, depth=1):
    """ Register resolver for given schema

    :param schema: uri schema
    :type schema: string
    :param resolver: Callable object that accept one parameter.

    Resolver interface::

      class Resolver(object):

         def __call__(self, uri):
             return content
    """
    info = config.DirectiveInfo(depth=depth)
    info.attach(
        config.Action(
            _register_uri_resolver, (schema, resolver),
            discriminator=(RESOLVER_ID, schema))
        )


def _register_uri_resolver(cfg, schema, resolver):
    cfg.get_cfg_storage(RESOLVER_ID)[schema] = resolver


class UriFactory(object):
    """ Uri Generator

    Example::

       >> uri = UriFactory('cms-content')

       >> uri()
       'cms-content:f73f3266fa15438e94cca3621a3f2dbc'

    """

    def __init__(self, schema):
        self.schema = schema

    def __call__(self):
        """ Generate new uri using supplied schema """
        return '%s:%s' % (self.schema, uuid.uuid4().get_hex())



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" vocabulary implementation from zope.schema """
from zope import interface
from ptah.form.interfaces import ITerm, IVocabulary


class SimpleTerm(object):
    """Simple tokenized term used by SimpleVocabulary."""
    interface.implements(ITerm)

    def __init__(self, value, token=None,
                 title=None, description=None):
        """Create a term for value and token. If token is omitted,
        str(value) is used for the token.  If title is provided,
        term implements ITitledTokenizedTerm.
        """
        self.value = value
        if token is None:
            token = value
        self.token = str(token)
        self.title = title
        self.description = description


class SimpleVocabulary(object):
    """Vocabulary that works from a sequence of terms."""
    interface.implements(IVocabulary)

    def __init__(self, *terms):
        """Initialize the vocabulary given a list of terms.

        The vocabulary keeps a reference to the list of terms passed
        in; it should never be modified while the vocabulary is used.
        """
        self.by_value = {}
        self.by_token = {}
        self._terms = terms
        for term in self._terms:
            if term.value in self.by_value:
                raise ValueError(
                    'term values must be unique: %s' % repr(term.value))
            if term.token in self.by_token:
                raise ValueError(
                    'term tokens must be unique: %s' % repr(term.token))
            self.by_value[term.value] = term
            self.by_token[term.token] = term

    @classmethod
    def from_items(cls, *items):
        """Construct a vocabulary from a list of (token, value) pairs. """
        terms = [cls.create_term(*rec) for rec in items]
        return cls(*terms)

    @classmethod
    def from_values(cls, *values):
        """Construct a vocabulary from a simple list. """
        terms = [cls.create_term(value) for value in values]
        return cls(*terms)

    @classmethod
    def create_term(cls, *args):
        """Create a single term from data."""
        return SimpleTerm(*args)

    def __contains__(self, value):
        try:
            return value in self.by_value
        except TypeError:
            # sometimes values are not hashable
            return False

    def get_term(self, value):
        try:
            return self.by_value[value]
        except KeyError:
            raise LookupError(value)

    def get_term_bytoken(self, token):
        try:
            return self.by_token[token]
        except KeyError:
            raise LookupError(token)

    def get_value(self, token):
        return self.get_term_bytoken(token).value

    def __iter__(self):
        return iter(self._terms)

    def __len__(self):
        return len(self.by_value)



""" snippet tests """
from zope import interface
from pyramid.httpexceptions import HTTPNotFound
from ptah import config, view
from ptah.view.snippet import ISnippet, Snippet, SnippetType, render_snippet

from base import Base


class TestSnippet(Base):

    def _setup_ptah(self):
        pass

    def test_snippettype_register(self):
        class ITestsnippet(interface.Interface):
            pass

        view.snippettype('test', Context)
        self._init_ptah()

        from ptah.view.snippet import STYPE_ID
        stypes = config.get_cfg_storage(STYPE_ID)

        self.assertTrue('test' in stypes)

        st = stypes['test']
        self.assertTrue(isinstance(st, SnippetType))
        self.assertEqual(st.name, 'test')
        self.assertEqual(st.context, Context)

    def test_snippet_register_no_stype(self):
        class TestSnippet(view.Snippet):
            pass

        view.register_snippet('unknown', Context, TestSnippet)
        self._init_ptah()

        snippet = self.registry.getMultiAdapter(
            (Context(), None), ISnippet, name='unknown')
        self.assertIsInstance(snippet, TestSnippet)

    def test_snippet_register(self):
        class TestSnippet(view.Snippet):
            def render(self):
                return 'test snippet'

        view.snippettype('test', Context)
        view.register_snippet('test', Context, TestSnippet)
        self._init_ptah()

        self.assertEqual(
            render_snippet('test', Context(), self.request), 'test snippet')

    def test_snippet_register_declarative(self):
        global TestSnippet

        view.snippettype('pt', Context)

        class TestSnippet(view.Snippet):
            view.snippet('pt')

            def render(self):
                return 'test'

        self._init_ptah()

        self.assertEqual(render_snippet('pt', Context(), self.request), 'test')

    def test_snippet_register_with_template(self):
        class TestSnippet(view.Snippet):
            pass

        def template(*args, **kw):
            keys = kw.keys()
            keys.sort()
            return '|'.join(keys)

        view.snippettype('test', Context)
        view.register_snippet('test', klass=TestSnippet, template = template)
        self._init_ptah()

        self.assertEqual(
            render_snippet('test', Context(), self.request),
            'context|request|view')

    def test_snippet_register_without_class(self):
        view.snippettype('test', Context)
        view.register_snippet('test', Context)
        self._init_ptah()

        snippet = self.registry.getMultiAdapter(
            (Context(), self.request), ISnippet, 'test')

        self.assertTrue(isinstance(snippet, Snippet))
        self.assertEqual(str(snippet.__class__),
                         "<class 'ptah.view.snippet.Snippet None'>")

    def test_snippet_register_with_not_Snippet_class(self):
        class TestSnippet(object):
            pass

        view.snippettype('test', Context)
        view.register_snippet('test', Context, TestSnippet)
        self._init_ptah()

        snippet = self.registry.getMultiAdapter(
            (Context(), self.request), ISnippet, 'test')

        self.assertTrue(isinstance(snippet, Snippet))
        self.assertTrue(isinstance(snippet, TestSnippet))

    def test_snippet_rendersnippet_not_found(self):
        self._init_ptah()

        self.assertRaises(
            HTTPNotFound,
            render_snippet, 'test', Context(), self.request)

    def test_snippet_render_with_exception(self):
        class TestSnippet(view.Snippet):
            def render(self):
                raise ValueError('Unknown')

        view.snippettype('test', Context)
        view.register_snippet('test', Context, TestSnippet)
        self._init_ptah()

        self.assertRaises(
            ValueError,
            render_snippet, 'test', Context(), self.request)

    def test_snippet_render_additional_params_to_template(self):
        class TestSnippet(view.Snippet):
            def update(self):
                return {'param1': 1, 'param2': 2}

        def template(*args, **kw):
            keys = kw.keys()
            keys.sort()
            return '|'.join(keys)

        view.snippettype('test', Context)
        view.register_snippet('test', klass=TestSnippet, template = template)
        self._init_ptah()

        self.assertTrue(
            'param1|param2|' in render_snippet('test', Context(), self.request))

    def test_snippet_View_snippet(self):
        class TestSnippet(view.Snippet):
            def render(self):
                return 'test snippet'

        view.snippettype('test', Context)
        view.register_snippet('test', Context, TestSnippet)
        self._init_ptah()

        base = view.View(None, self.request)

        # snippettype is string
        self.assertEqual(base.snippet('unknown', Context()), '')
        self.assertEqual(base.snippet('test', Context()), 'test snippet')

        # by default use view context
        base.context = Context()
        self.assertEqual(base.snippet('test'), 'test snippet')


    def test_snippet_View_snippet_with_error(self):
        class TestSnippet(view.Snippet):
            def render(self):
                raise ValueError('Error')

        view.snippettype('test', Context)
        view.register_snippet('test', Context, TestSnippet)
        self._init_ptah()

        base = view.View(None, self.request)
        self.assertEqual(base.snippet('test', Context()), '')


class Context(object):
    pass
